
#include "ContextContactsContainer.h"
#include <sendui.h>
#include <smut.h>
#include <SenduiMtmUids.h>
#include <txtrich.h>
#include <bautils.h>
#include <mmsconst.h>
#include <miutset.h>
#include <s32strm.h>
#include <akntitle.h> 
 
#include "doublelinebox.h"
#include "file_output_base.h"
#include <ContextContacts.rsg>
#include "contextcontacts.hrh"

#include <eikmenub.h>

#include "contextcontactsappui.h"
#include "presence_ui_helper.h"
#include "cbbsession.h"
#include "jabberdata.h"
#include "contextclientsession.h"

enum KEYCODES {
	JOY_LEFT = 0xF807,
	JOY_RIGHT = 0xF808,
	JOY_UP = 0xF809,
	JOY_DOWN = 0xF80A,
	JOY_CLICK = 0xF845,
	KEY_CALL = 0xF862,
	KEY_CANCEL = 0xF863,
	KEY_C = 0x0008
};

enum SCANCODES {
	CALL_SCAN = 0xC4,
	CANCEL_SCAN = 0xC5
};

#ifdef __S60V2__
#define USE_SKIN 1
#endif

#ifdef USE_SKIN
#include <AknsControlContext.h>
#include <AknsBasicBackgroundControlContext.h>
#include <AknsDrawUtils.h>
#endif

/*
 * Concepts:
 * !Listbox filtering!
 */

void CContextContactsContainer::ConstructL(const TRect& aRect, phonebook* book, Mfile_output_base * log, 
					   CAknIconArray * aIconlist, CAknView * view,
					   class CJabberData* aJabberData)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("ConstructL"));

	iJabberData=aJabberData;
	iFindBoxVisible=EFalse;
	iBBSession=BBSession()->CreateSubSessionL(0);
	{
		aLog = log;
		aView = view;

		CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
		CAknTitlePane* tp=(CAknTitlePane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidTitle));
		HBufC* t=CEikonEnv::Static()->AllocReadResourceL(R_CONTACTS_TITLE);
		tp->SetText(t);
	}
	
	{
		CreateWindowL();
		iCurrentVisibleSelection = new (ELeave) CArrayFixFlat<TInt>(100);
	}
	{
#ifdef USE_SKIN
		TRect rect=aRect;
		rect.Move( 0, -rect.iTl.iY );
		iBackground=CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,
			rect, EFalse );
#endif
	}
	{
		iIconlist = new (ELeave) CAknIconArray(30);
		for (TInt i = 0; i< aIconlist->Count();i++)
		{
			CGulIcon * ic, *icon_from; 
			icon_from= (*aIconlist)[i];
			ic=CGulIcon::NewL( icon_from->Bitmap(), icon_from->Mask());
			ic->SetBitmapsOwnedExternally(ETrue);
			iIconlist->AppendL( ic );
		}
	}

	{
		aBook=book;
		aBook->AddObserverL(this);

		iListBoxArray=CPresenceArray::NewL(book);
		iNameArray=CNameArray::NewL(book);
	}

	{
		iListbox=new (ELeave) doublelinebox(aBook, aLog);
		
		//iListbox->SetContainerWindowL(*this);
		iListbox->ConstructL(iNameArray, iListBoxArray, this, EAknListBoxMarkableList);
		iListbox->View()->SetMatcherCursor(EFalse);
		iListbox->ItemDrawer()->FormattedCellData()->SetIconArray(iIconlist);
		iIconlist=0;
		iListbox->SetListBoxObserver(this);
		iListbox->Model()->SetItemTextArray(iNameArray);
		iListbox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
		//FIX ME: use resource
		iListbox->View()->SetListEmptyTextL(_L("(No match)"));
		iListbox->CreateScrollBarFrameL(ETrue);
		iListbox->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EOff, 
			CEikScrollBarFrame::EAuto);
	}
	
	{
		CAknSearchField::TSearchFieldStyle style(CAknSearchField::ESearch );
		//CAknSearchField::TSearchFieldStyle style(CAknSearchField::EPlain );
		iFindBox = CreateFindBoxL( iListbox, iListbox->Model(), style );
		iFindBox->MakeVisible( ETrue );
		iFindBox->SetFocus(ETrue);

		SetRect(aRect);
		ActivateL();
	}
}

void CContextContactsContainer::ResetSearchField()
{
	if (iFindBox) iFindBox->ResetL();
}

CContextContactsContainer::~CContextContactsContainer()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("~CContextContactsContainer"));

	delete iBBSession;

	if (aBook) aBook->RemoveObserverL(this);
	delete iFindBox;
	delete iListbox;
	delete iCurrentVisibleSelection;

	delete iNameArray;
	delete iListBoxArray;

#ifdef USE_SKIN
	delete iBackground;
#endif
	delete iIconlist;
}

void CContextContactsContainer::SizeChanged()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("SizeChanged"));

	TInt findbox_height = 27;

	TRect lb_rect=Rect();
	if ( iListbox ) {
		if (iFindBoxVisible) {
			//if (iFindBox) iFindBox->SetFocus(ETrue);
			lb_rect.Resize(0, -(findbox_height));
		} else {
			lb_rect.Resize(0, 0);
		}
		iListbox->AdjustRectHeightToWholeNumberOfItems(lb_rect);
                iListbox->SetRect(lb_rect);
        }
	if (iFindBox) {
		TRect findbox_rect;
		if (iFindBoxVisible) {
			findbox_rect = TRect(
				TPoint(0, Rect().Height() - findbox_height), 
				TSize(Rect().Width(), findbox_height)
				);
		} else {
			findbox_rect = TRect(
				TPoint(0, Rect().Height() + findbox_height), 
				TSize(Rect().Width(), findbox_height)
				);
		}
		iFindBox->SetRect(findbox_rect);
	}
}

TInt CContextContactsContainer::CountComponentControls() const
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("CountComponentControls"));

	return 2; 
}

CCoeControl* CContextContactsContainer::ComponentControl(TInt aIndex) const
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("ComponentControl"));

	switch ( aIndex )
        {
		case 0:
			return iListbox;
			break;
		case 1:
			return iFindBox;
			break;
		default:
			return NULL;
        }
}
void CContextContactsContainer::Draw(const TRect& aRect) const
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("Draw"));

	CWindowGc& gc = SystemGc();

#ifndef USE_SKIN
	gc.SetPenStyle(CGraphicsContext::ENullPen);
	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
#else
	AknsDrawUtils::Background( AknsUtils::SkinInstance(), iBackground, gc, aRect );
#endif
}

void CContextContactsContainer::HandleControlEventL(
    CCoeControl* ,TCoeEvent )
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("HandleControlEventL"));
}


TKeyResponse CContextContactsContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("OfferKeyEventL"));

	// the search field / listbox combo
	TBool needRefresh( EFalse );
	TBool flagsOfPopup( EFalse );
	if ( aType != EEventKey ) /* Is not key event? */{
		// the find field catches the key presses it needs
		// but doesn't actually do the search. This
		// triggers it
		if (iFilter) iFilter->HandleOfferkeyEventL();

		if (iFindBox->TextLength()!= 0) {
			if (!iFindBoxVisible) { // not already showing
				iFindBoxVisible=ETrue;
				SizeChanged();
				DrawDeferred();
			}
		} else {
			if (iFindBoxVisible) { // already showing
				iFindBoxVisible=EFalse;
				SizeChanged();
				DrawDeferred();
			}
		}

		return EKeyWasNotConsumed;
        }
	if (aKeyEvent.iCode==JOY_LEFT) {
#ifdef GROUP_TAB
		return EKeyWasNotConsumed;
#else
		show_presence_details_current();
		return EKeyWasConsumed;
#endif
	} 
	
	if (aKeyEvent.iCode == JOY_RIGHT) {
#ifdef GROUP_TAB
		return EKeyWasNotConsumed;
#else
		show_presence_description_current();
		return EKeyWasConsumed;
#endif
	} 
	        
	if ( iListbox ) {
		if (aKeyEvent.iCode==JOY_DOWN || aKeyEvent.iCode == JOY_UP || aKeyEvent.iCode == JOY_CLICK){
			return iListbox->OfferKeyEventL(aKeyEvent, aType);
		} 
		if (aKeyEvent.iCode==KEY_CALL) {
			if ( CountVisibleItems() >0 ) {
				aView->HandleCommandL(EContextContactsCmdCall);
				return EKeyWasConsumed;
			}
			
		}
		if (aKeyEvent.iCode==KEY_C) {
			if (iFindBox) {
				if ( (iFindBox->TextLength() == 0) && (CountVisibleItems() >0)  ) {
					aView->HandleCommandL(EContextContactsCmdDelete);
					return EKeyWasConsumed;
				} 
			}
		}
		if (iFindBox) {
			TBool needs_refresh;
			return AknFind::HandleFindOfferKeyEventL( aKeyEvent, aType, this, iListbox, 
				iFindBox, EFalse, needs_refresh);
		}
		return iListbox->OfferKeyEventL(aKeyEvent, aType);
        } else {
		return EKeyWasNotConsumed;
        }
}

void CContextContactsContainer::before_change()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("before_change"));

	
	if (! iListbox ) {
		// still constructing
		return;
	}
	iCurrentIdx = get_current_idx();
	iTopIdx = get_current_top_idx();
}

void CContextContactsContainer::contents_changed(TInt ContactId, TBool aPresenceOnly)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("contents_changed"));

	if (!iListbox ) { return; }

	if (! aPresenceOnly ) {
		iListbox->HandleItemRemovalL();
		iFilter->HandleItemArrayChangeL();
		set_current_idx(iCurrentIdx);
		set_current_top_idx(iTopIdx);
	} else {
		TInt top = get_current_top_idx();
		TInt bottom = get_current_bottom_idx();
		TInt i;
		TBool redraw=EFalse;
		for (i=top; (i<=bottom) && !redraw; i++) {
			if (ContactId == aBook->GetContactId(i)) redraw=ETrue;
		}
		if (redraw) iListbox->DrawDeferred();
	}
}

void CContextContactsContainer::exiting()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("exiting"));

	aBook=0;
}

void CContextContactsContainer::HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("HandleListBoxEventL"));

	contact * c=0;

	if(aListBox == iListbox)
	{
		switch(aEventType)
		{
			case EEventEnterKeyPressed:
			{
#ifdef GROUP_TAB
				c = aBook->GetContact(get_current_idx());
				if (c == NULL || c->presence == NULL) {
					aView->HandleCommandL(EContextContactsCmdOpen);
				} else {
					aView->MenuBar()->SetMenuTitleResourceId(R_CONTEXTCONTACTS_MENUBAR_CLICK_ON_CONTACT);
					aView->MenuBar()->TryDisplayMenuBarL();
					aView->MenuBar()->SetMenuTitleResourceId(R_CONTEXTCONTACTS_MENUBAR_VIEW_CONTACTS);
				}
#else
				aView->HandleCommandL(EContextContactsCmdOpen);
#endif
				break;
			}
			default:
				break;
		}
	}
}

void CContextContactsContainer::show_presence_details_current()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("show_presence_details_current"));


	TInt current_index = get_current_idx();
	if (current_index < 0) return;

	contact * c = aBook->GetContact(current_index);
	if (c == NULL || c->presence == NULL) return;

	//FIX ME: check the CPbkContact engine status for first name/last name
	TBuf<128> name;

	if (c->last_name) name.Append(*(c->last_name));
	if (c->last_name && c->last_name->Length()>0 && c->first_name && c->first_name->Length()>0) {
		name.Append(_L(" "));
	}
	if (c->first_name) name.Append(*(c->first_name));
	
	if (aLog) 
	{
		aLog->write_time();
		aLog->write_to_output(_L("Showing pres. details of "));
		aLog->write_to_output(name);
		aLog->write_nl();
	}
	TBuf<50> nick;
	iJabberData->GetJabberNickL(c->id, nick);

	if (! c->show_details_in_list ) 
		SendLookupNotificationL(iBBSession, nick, c->presence);

	((CContextContactsAppUi *)(iEikonEnv->AppUi()))->DisplayPresenceDetailsL(name, (aBook->GetContact(current_index))->presence);
}

void CContextContactsContainer::show_presence_description_current()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("show_presence_description_current"));

	TInt current_index = get_current_idx();
	if (current_index < 0) return;

	contact * c = aBook->GetContact(current_index);
	if (c == NULL || c->presence == NULL) return;

	//FIX ME: check the CPbkCOntact engine status for first name/last name
	TBuf<128> name;

	if (c->last_name) name.Append(*(c->last_name));
	if (c->last_name && c->last_name->Length()>0 && c->first_name && c->first_name->Length()>0) {
		name.Append(_L(" "));
	}
	if (c->first_name) name.Append(*(c->first_name));
	
	if (aLog) 
	{
		aLog->write_time();
		aLog->write_to_output(_L("Showing pres. description of "));
		aLog->write_to_output(name);
		aLog->write_nl();
	}
	((CContextContactsAppUi *)(iEikonEnv->AppUi()))->DisplayPresenceDescriptionL(name, 
		(aBook->GetContact(current_index))->presence);
}


CAknSearchField* CContextContactsContainer::CreateFindBoxL(CEikListBox* aListBox, CTextListBoxModel* aModel, CAknSearchField::TSearchFieldStyle aStyle )
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("CreateFindBoxL"));

	CAknSearchField* findbox = NULL;

	if ( aListBox && aModel )
	{
		CAknFilteredTextListBoxModel* model = STATIC_CAST( CAknFilteredTextListBoxModel*, aModel );
		findbox = CAknSearchField::NewL( *this, aStyle, NULL, 30 );
		CleanupStack::PushL(findbox);
		model->CreateFilterL( aListBox, findbox );
		iFilter = model->Filter();

		iListbox->iPresenceModel->SetFilter(iFilter);
                CleanupStack::Pop(findbox); // findbox
        }
	return findbox;
}

void CContextContactsContainer::SizeChangedForFindBox()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("SizeChangedForFindBox"));

	if ( iListbox && iFindBox ) {
		//TInt findWindowResourceId( R_AVKON_FIND_PANE );
		//TInt listAreaResourceId( R_AVKON_LIST_GEN_PANE_X );
		//TInt findWindowParentId( R_AVKON_MAIN_PANE_PARENT_NONE );
		//TBool flagsOfPopup( ETrue ); 
		//AknFind::HandleFindSizeChanged( this, iListbox, iFindBox, flagsOfPopup, findWindowResourceId,
		//listAreaResourceId, R_AVKON_LIST_GEN_PANE_WITH_FIND_POPUP, findWindowParentId );
	}
}

void CContextContactsContainer::MarkCurrentItemL()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("MarkCurrentItemL"));

	TInt real_idx = get_current_idx();
	iListbox->View()->ToggleItemL(iListbox->CurrentItemIndex());

	TInt found = -1;
	for (int i=0; (i< iCurrentVisibleSelection->Count()) && (found == -1) ;i++)
	{
		if (iCurrentVisibleSelection->At(i) == real_idx)
		{
			found = i;
		}
	}
	if (found != -1) // found!
	{
		iCurrentVisibleSelection->Delete(found);
	}
	else
	{
		iCurrentVisibleSelection->AppendL(real_idx);
	}
}

void CContextContactsContainer::MarkAllL()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("MarkAllL"));

	// matk all the visible items !
	for (TInt j = 0; j< iFilter->FilteredNumberOfItems(); j++)
	{
		iListbox->View()->SelectItemL(j);
		TInt real_idx = iFilter->FilteredItemIndex(j);
		int found = -1;
		for (int i=0; (i< iCurrentVisibleSelection->Count()) && (found == -1);i++)
		{
			if (iCurrentVisibleSelection->At(i) == real_idx)
			{
				found = i;
			}
		}
		if (found == -1) iCurrentVisibleSelection->AppendL(real_idx);
	}
}

void CContextContactsContainer::UnmarkAll()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("UnmarkAll"));

	iListbox->View()->ClearSelection();
	// unmark everything
	iCurrentVisibleSelection->Reset();
	iCurrentVisibleSelection->Compress();
}

TInt CContextContactsContainer::CountVisibleItems()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("CountVisibleItems"));

	return iFilter->FilteredNumberOfItems();
}

CArrayFix<TInt> * CContextContactsContainer::get_visible_selection()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("get_visible_selection"));

	// when an action ha been made on marked items, they return to unmarked state.
	if ( iListbox->View()->SelectionIndexes()->Count() == 0 ) {
		iCurrentVisibleSelection->Reset();
	}
	return iCurrentVisibleSelection;
}

TInt CContextContactsContainer::get_current_idx()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("get_current_idx"));

	if (iFilter->FilteredNumberOfItems() == 0) return -1;
	TInt real_idx = iFilter->FilteredItemIndex(iListbox->CurrentItemIndex());
	return real_idx;
}

TInt CContextContactsContainer::get_current_top_idx()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("get_current_top_idx"));

	if (iFilter->FilteredNumberOfItems() == 0) return -1;
	TInt real_idx = iFilter->FilteredItemIndex(iListbox->TopItemIndex());
	return real_idx;
}

TInt CContextContactsContainer::get_current_bottom_idx()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("get_current_bottom_idx"));

	if (iFilter->FilteredNumberOfItems() == 0) return -1;
	TInt real_idx = iFilter->FilteredItemIndex(iListbox->BottomItemIndex());
	return real_idx;
}

void CContextContactsContainer::set_current_idx(TInt real_idx)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("set_current_idx"));

	if (real_idx == -1) return;
	if (iFilter->FilteredNumberOfItems() < 1) return;

	TBuf8<20> sel;

	TInt idx;
	if (real_idx > iFilter->FilteredNumberOfItems() ) {
		sel=_L8("idx = 0;");
		idx = 0;	
	} else if (real_idx == iFilter->FilteredNumberOfItems()) {
		sel=_L8("idx = real_idx-1;");
		idx = real_idx-1;
	} else {
		sel=_L8("idx = iFilter->");
		idx = iFilter->VisibleItemIndex(real_idx);
	}
	
	if (idx < 0) return;

	{
		TBuf8<15> ib; ib.AppendNum(idx);
		CALLSTACKITEM_N(sel, ib);
		iListbox->View()->SetCurrentItemIndex(idx);
		iListbox->DrawDeferred();
	}
}

void CContextContactsContainer::set_current_top_idx(TInt real_idx)
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("set_current_top_idx"));

	if (real_idx == -1) return;
	if (iFilter->FilteredNumberOfItems() < 1) return;

	TInt idx;
	if (real_idx > iFilter->FilteredNumberOfItems() )
	{
		idx = 0;	
	}
	else if (real_idx == iFilter->FilteredNumberOfItems())
	{
		idx = real_idx-1;
	}
	else
	{
		idx = iFilter->VisibleItemIndex(real_idx);
	}
	if (idx < 0) return;
	iListbox->View()->SetTopItemIndex(idx);
	//iListbox->View()->SetCurrentItemIndex(idx);
	iListbox->DrawDeferred();
}

TBool CContextContactsContainer::IsCurrentMarked()
{
	CALLSTACKITEM_N(_CL("CContextContactsContainer"), _CL("IsCurrentMarked"));

	TInt idx = get_current_idx();

	const CListBoxView::CSelectionIndexArray* selectionArray = iListbox->View()->SelectionIndexes();

	TBool found = EFalse;
	for (TInt i = 0; (i < selectionArray->Count()) && (found == EFalse); i++)
	{
		if ((*selectionArray)[i] == idx) 
		{ 
			found = ETrue;
		}
	}
	return found;
}
