#include "break.h"
#include "cm_tags.h"
#include "db.h"
#include "symbian_auto_ptr.h"

class CTagStorageImpl : public CTagStorage, public MDBStore {
public:
	CTagStorageImpl(CDb& aDb) : MDBStore(aDb.Db()) { }

	enum TColumns { 
		ETag=1, 
		ELastUse, 
		EUseCount };
	void ConstructL() {
		TInt cols[]= { 
			EDbColText,	// tag
			EDbColDateTime, // last use
			EDbColInt32, // usecount
			-1
		};
		TInt col_flags[]={ 0, 0, 0 };
		TInt idxs[]= { ETag, -2, ELastUse, -2, EUseCount, -1 };

		_LIT(KTags, "TAGS");
		SetTextComparison(EDbCompareCollated);	
		MDBStore::ConstructL(cols, idxs, false, KTags, ETrue, col_flags);

		iPos=-1;
	}
	TInt iPos;

	TInt MdcaCountL() {
		return iTable.CountL();
	}
	virtual TInt MdcaCount() const {
		CTagStorageImpl* i=(CTagStorageImpl*) this;
		TInt count=0;
		CC_TRAPD(err, count=i->MdcaCountL()); 
		return count;
	}

	void GoToItemL(TInt aIndex) {
		if (aIndex<0) User::Leave(KErrArgument);
		if (iPos==-1) {
			if (!iTable.FirstL()) User::Leave(KErrNotFound);
			iPos=0;
		}
		while (iPos<aIndex) {
			if (!iTable.NextL()) {
				iPos=-1;
				User::Leave(KErrNotFound);
			}
			++iPos;
		}
		while (iPos>aIndex) {
			if (!iTable.PreviousL()) {
				iPos=-1;
				User::Leave(KErrNotFound);
			}
			--iPos;
		}
	}
	virtual TPtrC16 MdcaPointL(TInt aIndex) {
		GoToItemL(aIndex);
		iTable.GetL();
		return iTable.ColDes(ETag);
	}

	virtual void DeleteItemL(TInt aIndex) {
		GoToItemL(aIndex);
		iPos=-1;
		DeleteL();
	}

	TBuf<52> iBuf;
	virtual TPtrC16 MdcaPoint(TInt aIndex) const {
		CTagStorageImpl* i=(CTagStorageImpl*) this;
		i->iBuf=_L("0\t");
		TPtrC16 p(0, 0);
		CC_TRAPD(err, p.Set(i->MdcaPointL(aIndex))); 
		if (err!=KErrNone) i->iPos=-1;
		i->iBuf.Append(p);
		return iBuf.Mid(0);
	}

	virtual TBool TagExists(const TDesC& aString) {
		TBuf<50> tag;
		if (aString.Length()>50) tag=aString.Left(50);
		else tag=aString;
		tag.Trim();
		if (tag.Length()==0) return EFalse;

		iPos=-1;
		TDbSeekKey rk(tag);
		if ( iTable.SeekL(rk) ) {
			return ETrue;
		}
		return EFalse;
	}

	virtual void AddSingleStringL(const TDesC& aString) {
		TBuf<50> tag;
		if (aString.Length()>50) tag=aString.Left(50);
		else tag=aString;
		tag.Trim();
		if (tag.Length()==0) return;

		iPos=-1;
		TDbSeekKey rk(tag);
		if ( iTable.SeekL(rk) ) {
			iTable.UpdateL();
			iTable.SetColL(EUseCount, iTable.ColInt(EUseCount)+1);
		} else {
			iTable.InsertL();
			iTable.SetColL(EUseCount, 1);
		}
		iTable.SetColL(ETag, tag);
		iTable.SetColL(ELastUse, GetTime());
		PutL();
	}
	virtual void AddFromStringAsNecessaryL(const TDesC& aString) {
		CALLSTACKITEM_N(_CL("CTagStorageImpl"), _CL("AddFromStringAsNecessaryL"));
		if (aString.Length()==0) return;

		TInt start_pos=0, colon_pos;
		colon_pos=aString.Locate(':');
		while (colon_pos!=KErrNotFound) {
			if (start_pos<colon_pos-1) {
				AddSingleStringL(aString.Mid(start_pos, colon_pos-start_pos));
			}
			if (colon_pos==aString.Length()-1) return;
			start_pos=colon_pos+1;
			colon_pos=aString.Mid(start_pos).Locate(':');
			if (colon_pos!=KErrNotFound) colon_pos+=start_pos;
		}
		AddSingleStringL(aString.Mid(start_pos, aString.Length()-start_pos));
	}

	friend CTagStorage;
	friend auto_ptr<CTagStorageImpl>;
};

EXPORT_C CTagStorage* CTagStorage::NewL(class CDb& aDb)
{
	auto_ptr<CTagStorageImpl> ret( new (ELeave) CTagStorageImpl(aDb) );
	ret->ConstructL();
	return ret.release();
}