#include "break.h"
#include "ContextMediaAppContainer.h"
#include <akntitle.h>
#include <aknnavi.h>
#include "ContextMediaAppView2.h"
#include "ContextMediaAppView.h"
#include <aknviewappui.h> 
#include "contextmediaui.hrh"
#include <contextmediaui.rsg>
#include <txtrich.h>
#include "cm_post.h"
#include "cl_settings.h"
#include <avkon.rsg>
#include <aknnotewrappers.h>
#include <eiksbfrm.h>
#include <akncontext.h>
#include <bautils.h>
#include <plpvariant.h>
#include "Contextnotifyclientsession.h"
#include "cm_tags.h"
#include "cm_taglist.h"
#include "cm_autotaglist.h"
#include "bberrorinfo.h"

#ifdef __S60V2__
#define USE_SKIN 1
#endif

#ifdef USE_SKIN
#include <AknsControlContext.h>
#include <AknsBasicBackgroundControlContext.h>
#include <AknsDrawUtils.h>
#endif

/*
 * We want to load jpeg files as quickly as possible.
 * Using the CMdaImageFileToBitmapUtility directly is very slow,
 * so we use CPAlbImageViewerBasic which is a lot faster (it
 * uses CMdaImageFileToBitmapUtility internally, so it must
 * be giving some special parameters to it we are not privy to).
 * The problem with CPAlbImageViewerBasic is that the loading
 * isn't asynchronous to the caller: it all happens in one
 * function call. However, the LoadImageL actually starts another
 * active scheduler loop! So we may get user-interface events while
 * in the call, introducing re-entrancy. Handling of that 
 * re-entrancy is done in the DisplayPicture and DisplayPictureInnerL.
 *
 */

// Some display constants

#define KTextColor TRgb(0,0,0)
#define KFocusedBgColor TRgb(205,205,255)
#define KUnfocusedBgColor TRgb(255, 255, 255)
#define KTagsBgColor TRgb(255, 255, 255)

#define KPictureSize TSize(174,131)
#define KPictureSizeStandAlone TSize(160, 120)
#define KVideoSize   TSize(174,139)
#define KVideoSize2  TSize(176,143)

#define KPictureRectangle TRect(TPoint(1,5), KPictureSize)
#define KPictureRectangleStandAlone TRect(TPoint(1+7,9), KPictureSizeStandAlone)
#define KVideoRectangle TRect(TPoint(1,1), KVideoSize)
#define KVideoRectangle2 TRect(TPoint(0,0), KVideoSize2)

#ifdef __WINS__
	_LIT(KMbmVideo, "z:\\system\\data\\contextmediaui_video.mbm");
	_LIT(KMbmAudio, "z:\\system\\data\\contextmediaui_audio.mbm");
	_LIT(KMbmUnknown, "z:\\system\\data\\contextmediaui_unknown.mbm");
#else
	_LIT(KMbmVideo, "c:\\system\\data\\contextmediaui_video.mbm");
	_LIT(KMbmAudio, "c:\\system\\data\\contextmediaui_audio.mbm");
	_LIT(KMbmUnknown, "c:\\system\\data\\contextmediaui_unknown.mbm");
#endif

EXPORT_C TInt LoadCmuiResourceL()
{
	CEikonEnv* env=CEikonEnv::Static();
#ifndef __WINS__
	TFileName resfile=_L("c:\\System\\data\\contextmediaui.rsc");
	if (! BaflUtils::FileExists(env->FsSession(), resfile) ) {
		resfile.Replace(0, 1, _L("e"));
	}
#else
	TFileName resfile=_L("z:\\System\\data\\contextmediaui.rsc");
	BaflUtils::NearestLanguageFile(env->FsSession(), resfile);
#endif
	return env->AddResourceFileL(resfile);
}

class OurOwnRichTextEditor : public CEikRichTextEditor
{
public:
	OurOwnRichTextEditor(HBufC * aDefaultValue): iDefaultValue(aDefaultValue) {}
	inline ~OurOwnRichTextEditor() {}
	void PrepareForFocusLossL();
	void PrepareForFocusGainL();
	void HandleTextChangedL();
	void Reset();
	void Zero();
	void AppendParagraph(const TDesC& aText, TRgb aColor, CParaFormat::TAlignment aAlignment);
private:
	HBufC * iDefaultValue;
};

void GetIMEI(TDes& aInto)
{
#ifndef __WINS__
	TPlpVariantMachineId machineId;
	PlpVariant::GetMachineIdL(machineId);
	aInto=machineId;
#else
	// Return a fake IMEI when working on emulator
	_LIT(KEmulatorImsi, "244050000000000");
	aInto=KEmulatorImsi;
#endif
}

void CopyFbsBitmapL(CFbsBitmap * orig, CFbsBitmap * dest)
{
	TSize s = orig->SizeInPixels();
	User::LeaveIfError(dest->Create(s, orig->DisplayMode()));

	TBitmapUtil orig_util(orig);
	TBitmapUtil dest_util(dest);
	orig_util.Begin(TPoint(0,0)); 
	dest_util.Begin(TPoint(0,0), orig_util); 

	TInt xPos;
	for (TInt yPos=0;yPos<s.iHeight;yPos++) {
		orig_util.SetPos(TPoint(0,yPos));
		dest_util.SetPos(TPoint(0,yPos));
		for (xPos=0;xPos<s.iWidth;xPos++) {
			dest_util.SetPixel(orig_util);
			orig_util.IncXPos();
			dest_util.IncXPos();
		}
	}
	orig_util.End();
	dest_util.End();
}

void AddBusyToBitmap(CFbsBitmap * bitmap)
{
	TRgb target_colour=KRgbRed;
	if (bitmap->DisplayMode() == EGray2) {
		target_colour=KRgbBlack;
	}

	TDisplayMode m = bitmap->DisplayMode();
	if (m>=EColor4K) m=EColor256;
	auto_ptr<CPalette> p (CPalette::NewDefaultL(m));
	TInt colour_idx = p->NearestIndex(KRgbRed);

	TBitmapUtil util(bitmap);
	util.Begin(TPoint(0,0));
	TInt xPos,yPos;
	for (yPos=5; yPos<20; yPos++) {
		for (xPos=5; xPos<20; xPos++) {
			util.SetPos(TPoint(xPos,yPos));
			util.SetPixel(colour_idx);
			util.IncXPos();
		}
	}
	util.End();
}

//-------------------------------------------------------------------------------------

CContextMediaAppGeneralContainer::CContextMediaAppGeneralContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
				CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView): 
			iStorage(aStorage), 
			iCMNetwork(aCMNetwork),
			iNode(aNode),
			iSort(aSort), 
			iOrder(aOrder),
			iView (aView){}

void CContextMediaAppGeneralContainer::BaseConstructL()
{
	iVideoMbmPath=KMbmVideo();
	iAudioMbmPath=KMbmAudio();
	iUnknownMbmPath=KMbmUnknown();
	if (! BaflUtils::FileExists(Fs(), iVideoMbmPath))	iVideoMbmPath.Replace(0, 1, _L("e"));
	if (! BaflUtils::FileExists(Fs(), iAudioMbmPath))	iAudioMbmPath.Replace(0, 1, _L("e"));
	if (! BaflUtils::FileExists(Fs(), iUnknownMbmPath)) 	iUnknownMbmPath.Replace(0, 1, _L("e"));
}

void CContextMediaAppGeneralContainer::GetVideoMbmPath(TFileName &aFilename)
{
	aFilename.Copy(iVideoMbmPath);
}

void CContextMediaAppGeneralContainer::GetAudioMbmPath(TFileName &aFilename)
{
	aFilename.Copy(iAudioMbmPath);
}

void CContextMediaAppGeneralContainer::GetUnknownMbmPath(TFileName &aFilename)
{
	aFilename.Copy(iUnknownMbmPath);
}


CContextMediaAppGeneralContainer::~CContextMediaAppGeneralContainer()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppGeneralContainer"), _CL("~CContextMediaAppGeneralContainer"));
}

EXPORT_C CContextMediaAppListboxContainer::CContextMediaAppListboxContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
				 CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView,
				 TBool aStandAlone) : 
				CContextMediaAppGeneralContainer(aCMNetwork, aStorage, aNode, aSort, aOrder, aView),
					iStandAlone(aStandAlone) { }

TBool CContextMediaAppListboxContainer::IsCurrentPlayableMedia()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("IsCurrentPlayableMedia"));

	return EFalse;
}

TBool CContextMediaAppListboxContainer::IsLoadingMedia()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("IsLoadingMedia"));

	return EFalse;
}

void CContextMediaAppListboxContainer::expired(CBase * /*Source*/)
{
	if (iListbox) {
		iListbox->HandleItemAdditionL();
		iListbox->DrawNow();
	}
}

void CContextMediaAppListboxContainer::ConstructL(const TRect& aRect)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("ConstructL"));

	BaseConstructL();
	CreateWindowL();

	iTimer = CTimeOut::NewL(*this);

	{
#ifdef USE_SKIN
		TRect rect=aRect;
		rect.Move( 0, -rect.iTl.iY );
		iBackground=CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,
			rect, EFalse );
#endif
	}

	// Bufs from resource
	iNoTitleBuf = CEikonEnv::Static()->AllocReadResourceL(R_NO_TITLE);
	iNoItemBuf = CEikonEnv::Static()->AllocReadResourceL(R_NO_ITEM);
	iNoVisibleItemBuf = CEikonEnv::Static()->AllocReadResourceL(R_NO_ITEM_VISIBLE);
	iNewThreadBuf = CEikonEnv::Static()->AllocReadResourceL(R_NEW_THREAD);
        
	// Create array and necessary additions
	CContextMediaArray::TAdditionalItem aItem = CContextMediaArray::ENone;
	if (iView->Id()==KThreadViewId) {
		aItem = CContextMediaArray::EAddReply;
	} else if (iView->Id()==KThreadsByDatePromptId){
		aItem = CContextMediaArray::EUseVCode;
	}
	iMediaTextArray = CContextMediaArray::NewL(iCMNetwork, iStorage, iNode, iSort, iOrder, *this, aItem,
		iStandAlone);

	if (!iStandAlone) {
		// CREATE A NEW TITLE PANEL IF NECESSARY
		CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
		CAknNavigationControlContainer *np = (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi)); 
		CLocalNotifyWindow* tp=CLocalNotifyWindow::Global();
		CAknContextPane *cp =(CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));

		if (iView->Id()==KThreadsByDatePromptId) {
			np->PushDefaultL(ETrue);	
			HBufC * t = iEikonEnv->AllocReadResourceL(R_SELECT_THREAD);
			tp->SetTitleText(t);
		} else if (iNode != CPostStorage::RootId()) {
			np->PushDefaultL(ETrue);
			TInt count = ((MDesCArray*)iMediaTextArray)->MdcaCount();
			CCMPost * aPost =0;
			if (count) aPost = iMediaTextArray->GetPostAt(count-1);

			if (aPost) {
				if (aPost->iBodyText->Value().Length()==0) {
					tp->SetTitleText((*iNoTitleBuf).AllocL());
				} else {
					tp->SetTitleText(aPost->iBodyText->Value().AllocL());
				}
				CGulIcon * icon = iStorage.IconArray()->At(aPost->GetThumbnailIndex());
				CFbsBitmap * aBitmap = new (ELeave) CFbsBitmap;
				aBitmap->Duplicate(icon->Bitmap()->Handle());
				cp->SetPicture(aBitmap);
			} else {
				// New thread!
				CGulIcon * icon = iStorage.IconArray()->At(KNewThreadIconIndex); 
				CFbsBitmap * aBitmap = new (ELeave) CFbsBitmap;
				aBitmap->Duplicate(icon->Bitmap()->Handle());
				cp->SetPicture(aBitmap);
				tp->SetTitleText((*iNewThreadBuf).AllocL());
			}
		} else {
			cp->SetPictureToDefaultL();
			HBufC * t = iEikonEnv->AllocReadResourceL(R_TITLE);
			tp->SetTitleText(t);
		}
	}

	iListbox = new (ELeave) CContextMediaBox(iMediaTextArray, iStandAlone);
	//iListbox->SetMopParent(this);
	//iListbox->SetContainerWindowL(*this);
	iListbox->ConstructL(this, EAknListBoxSelectionList|EAknListBoxLoopScrolling );
	iListbox->View()->SetMatcherCursor(EFalse);
	iListbox->ItemDrawer()->FormattedCellData()->SetIconArray(iStorage.IconArray());
	iListbox->Model()->SetItemTextArray(iMediaTextArray);

	if (!iStandAlone) 
		iListbox->SetItemHeightL(48);
	else
		iListbox->SetItemHeightL(20);

	iListbox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
	iListbox->SetListBoxObserver(this);
	iListbox->CreateScrollBarFrameL(ETrue);
	iListbox->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);

	iNoItemLabel = new (ELeave) CEikLabel;
	iNoItemLabel->SetPosition(TPoint(20,20));
	
	SetRect(aRect);
	ActivateL();
}


void CContextMediaAppListboxContainer::PostEvent(CCMPost* /*aParent*/, CCMPost* aChild, TEvent aEvent)
{
	if (aEvent==EChildAdded) {
		TInt count = iMediaTextArray->MdcaCount();
		if ((iView->Id()==KThreadViewId) && (aChild==iMediaTextArray->GetPostAt(count-1))) {
			CLocalNotifyWindow* tp=CLocalNotifyWindow::Global();
			if (aChild->iBodyText->Value().Length()==0) {
				tp->SetTitleText((*iNoTitleBuf).AllocL());
			} else {
				tp->SetTitleText(aChild->iBodyText->Value().AllocL());
			}
			iListbox->HandleItemAdditionL();
			iListbox->DrawNow();
		} else {
			iTimer->Wait(1);
		}		
	} else if ((aEvent==EPostVisible) || (aEvent==EPostUpdated) || (aEvent==ELastPostChanged)) {
		iTimer->Wait(1);
	} else if (aEvent==EThumbnailLoaded ) {
		TInt count = iMediaTextArray->MdcaCount();
		if ((iView->Id()==KThreadViewId) && (aChild==iMediaTextArray->GetPostAt(count-1))) {
			if (!iStandAlone) {
				//change current status pane bitmap
				CGulIcon * icon = iStorage.IconArray()->At(aChild->GetThumbnailIndex());
				CFbsBitmap * aBitmap = new (ELeave) CFbsBitmap;
				aBitmap->Duplicate(icon->Bitmap()->Handle());
				CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
				CAknContextPane *cp =(CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));
				cp->SetPicture(aBitmap);
				cp->DrawNow();
			}
		} 
		iListbox->DrawNow();
	} else {
		iListbox->DrawDeferred();
	}
}


CContextMediaAppListboxContainer::~CContextMediaAppListboxContainer()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("~CContextMediaAppListboxContainer"));
	
	delete iTimer;

	if (iView->Id()==KThreadsByDatePromptId) {
		CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
		CAknNavigationControlContainer *np =0;

		if (sp) np=(CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi)); 
		if (np) np->Pop(NULL);

	} else if (iNode != CPostStorage::RootId()) {
		CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
		CAknContextPane *cp =0;
		CAknNavigationControlContainer* np =0;
		if (sp) cp = (CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));
		if (cp) cp->SetPictureToDefaultL();
		
		if (sp) np = (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi));
		if (np) np->Pop(NULL);
		
	}
	if (iListbox) {
		iListbox->MakeVisible(EFalse);
		iListbox->ItemDrawer()->FormattedCellData()->SetIconArray(0);
	}
	
	delete iListbox;
	delete iNoItemLabel;

	delete iMediaTextArray;
	
	delete iNoTitleBuf;
	delete iNoItemBuf;
	delete iNoVisibleItemBuf;
	delete iNewThreadBuf;
#ifdef USE_SKIN
	delete iBackground;
#endif
}

void CContextMediaAppListboxContainer::SizeChanged()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("SizeChanged"));

	if (iListbox) {
		TRect lb_rect=Rect();
		iListbox->AdjustRectHeightToWholeNumberOfItems(lb_rect);
                iListbox->SetRect(lb_rect);
	}
}

TInt CContextMediaAppListboxContainer::CountComponentControls() const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("CountComponentControls"));

	return 2; 
}

CCoeControl* CContextMediaAppListboxContainer::ComponentControl(TInt aIndex) const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("ComponentControl"));

	switch ( aIndex ) {
		case 0:	 return iListbox;
		case 1:  return iNoItemLabel;
		default: return NULL;
        }
}

void CContextMediaAppListboxContainer::Draw(const TRect& aRect) const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("Draw"));

	// handling visibility of listbox and default empty text
	if (iListbox->Model()->NumberOfItems() == 0) {
		if (iStorage.HasHiddenThreads()) {
			iNoItemLabel->SetTextL(*iNoVisibleItemBuf);
		} else {
			iNoItemLabel->SetTextL(*iNoItemBuf);
		}
		iNoItemLabel->SetSize(iNoItemLabel->MinimumSize());
		iNoItemLabel->SetPosition(TPoint( (176-iNoItemLabel->MinimumSize().iWidth)/2, 50));
	} else {
		iNoItemLabel->SetSize(TSize(0,0));
	}

	CWindowGc& gc = SystemGc();

#ifndef USE_SKIN
	gc.SetPenStyle(CGraphicsContext::ENullPen);
	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
#else
	AknsDrawUtils::Background( AknsUtils::SkinInstance(), iBackground, gc, aRect );
#endif
}

void CContextMediaAppListboxContainer::HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("HandleListBoxEventL"));

	if(aListBox == iListbox) {
		switch(aEventType) {
			case EEventEnterKeyPressed: {
				if (get_current_post()) {
					iView->HandleCommandL(EcontextmediaappCmdOpenItem);
				}
				break;
			}
			default:
				break;
		}
	}
}

void CContextMediaAppGeneralContainer::mark_as_read()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppGeneralContainer"), _CL("mark_as_read"));

	CCMPost * aPost = get_current_post();
	if (aPost) iStorage.MarkAsRead(aPost);
}

TKeyResponse CContextMediaAppListboxContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("OfferKeyEventL"));

	if ((aKeyEvent.iCode == EKeyLeftArrow )|| (aKeyEvent.iCode == EKeyRightArrow )) {
		if (iView->Id()==KThreadsByDatePromptId) {
			return EKeyWasConsumed;
		} else {
			return EKeyWasNotConsumed;
		}
	}

	if ( iListbox ) {
                if ((aKeyEvent.iCode ==EKeyOK) && !get_current_post() ) {
			if (iView->Id()==KThreadsByDatePromptId) {
				iView->HandleCommandL(EAknSoftkeySelect);
				return EKeyWasConsumed;
			} else {
				if (iListbox->Model()->NumberOfItems()!=0) iView->HandleCommandL(EcontextmediaappCmdReply);
				return EKeyWasConsumed;
			}
		} else if (aKeyEvent.iCode == EKeyYes){
			if (iView->Id()==KThreadViewId) iView->HandleCommandL(EcontextmediaappCmdReply);
			return EKeyWasConsumed;
		} else {
			return iListbox->OfferKeyEventL(aKeyEvent, aType);
		}
	} else {
		return EKeyWasNotConsumed;
        }
}

TInt CContextMediaAppListboxContainer::get_current_idx()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("get_current_idx"));

	if (iListbox->Model()->NumberOfItems()) {
		return iListbox->CurrentItemIndex();
	}
	return -1;
}

TInt CContextMediaAppListboxContainer::get_top_idx()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("get_top_idx"));

	if (this->iListbox->Model()->NumberOfItems()) {
		return iListbox->TopItemIndex();
	}
	return -1;
}

TInt CContextMediaAppListboxContainer::get_item_count()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_item_count"));

	return iListbox->Model()->NumberOfItems();
}


void CContextMediaAppListboxContainer::set_current_idx(TInt idx)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("set_current_idx"));

	if (iListbox && (idx!=-1) && (iListbox->Model()->NumberOfItems()>idx)) {
		iListbox->SetCurrentItemIndex(idx);
	}
}

void CContextMediaAppListboxContainer::set_top_idx(TInt idx)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("set_top_idx"));

	if (iListbox && (idx!=-1) && (iListbox->Model()->NumberOfItems()>idx)) {
		iListbox->SetTopItemIndex(idx);
	}
}


CCMPost * CContextMediaAppListboxContainer::get_current_post()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppListboxContainer"), _CL("get_current_post"));

	TInt idx = get_current_idx();
	
	if (idx>-1) return iMediaTextArray->GetPostAt(idx);
	return 0;
}

//_---------------------------------------------------------------------------------
//_---------------------------------------------------------------------------------
//_---------------------------------------------------------------------------------
//_---------------------------------------------------------------------------------
//_---------------------------------------------------------------------------------

EXPORT_C CContextMediaAppPostContainer::CContextMediaAppPostContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
				 CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView, 
				TBool aStandAlone, CTagStorage* aTagStorage,
				 const TDesC& aFileName, MBBDataFactory * aFactory,
				 CAknIconArray *aTagIcons, MAskForNames* aAskForNames) : 
CContextMediaAppGeneralContainer(aCMNetwork, aStorage, aNode,aSort,aOrder,aView), iFileName(aFileName), 
	iFactory(aFactory), iStandAlone(aStandAlone), iTagStorage(aTagStorage), iTagIcons(aTagIcons),
	iAskForNames(aAskForNames) {}

void CContextMediaAppPostContainer::ConstructL(const TRect& aRect )
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("ConstructL"));

	BaseConstructL();

	iLoadingMedia=EFalse;
	iPlayingMedia=EFalse;
	iMediaReady=EFalse;
	iMediaBrowsingOn = (iFileName.Length()==0);

	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("ConstructL1"));
		iVideoEngine = CVideoEngine::NewL();
		iAudioEngine = CAudioEngine::NewL();
		iDisplayTimeOut = CTimeOut::NewL(*this, CActive::EPriorityIdle);
		iVolumeDisplayTimeOut = CTimeOut::NewL(*this);
		iPostArray = new (ELeave) CArrayFixFlat<TUint>(50);
		iControls=new (ELeave) CArrayPtrFlat< CCoeControl >(10);
	}

// array of post ids
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_posts"));
		if (iView->Id()==KPostViewId) {
			TBool ok= EFalse;
			if (iNode != iStorage.RootId() ) {
				CC_TRAP(iLoadError, iPost = iStorage.GetByPostIdL(this, iNode)); 
				if (iLoadError==KErrNone) {
					iParentPost = iStorage.GetByPostIdL(this, iPost->iParentId());
					ok = iStorage.FirstL(iPost->iParentId(), iSort, iOrder, EFalse);
				}
			} else {
				iPost=iParentPost=0;
				ok=iStorage.FirstAllL();
				if (ok) iPost=iStorage.GetCurrentL(this);
			}
			int i=0;
			while (ok) {
				TUint idx=iStorage.GetCurrentIndexL();
				iPostArray->AppendL(idx);
				if (idx == iPost->iLocalDatabaseId()) {
					iCurrentPostIndex = i;
				}
				i++;
				ok = iStorage.NextL();
			}
		} else {
			if (iMediaBrowsingOn) {
				// we get posts from the thread mediapool
				TBool ok = iStorage.FirstL(CPostStorage::MediaPoolId(), iSort, iOrder, ETrue /*important!*/);
				while (ok) {
					TUint idx=iStorage.GetCurrentIndexL();
					iPostArray->AppendL(idx);
					ok = iStorage.NextL();
				}
			} else {

				// we create a single post from the given iFileName
				TTime t; t=GetTime();
				auto_ptr<CFbsBitmap> bm;
				refcounted_ptr<CCMPost>  aPost(CCMPost::NewL(iFactory, 0));
				aPost->iParentId = CPostStorage::MediaPoolId(); 
				aPost->iMediaUrl = iFileName; 
				aPost->iMediaFileName = iFileName;
				TBuf<40> mime; GetMimeTypeL(iFileName, mime);
				aPost->iContentType() = mime;
				aPost->iPostId = CPostStorage::AddMeToMediaPoolId();
				aPost->iUnreadCounter = 1; // IMPORTANT, cause a read post doesn't appear in the list
				aPost->iTimeStamp = t;

				aPost->iPresence()=iPresence; iPresence=0;

				TBuf<100> buf;
				if ( Settings().GetSettingL(SETTING_PUBLISH_AUTHOR, buf ) && buf.Length() != 0) {
					aPost->iSender.iName().Append(buf);
				}
				if ( Settings().GetSettingL(SETTING_UPLOAD_TAG, buf) ) {
					aPost->iTag->Append(buf);
				}
				if ( Settings().GetSettingL(SETTING_JABBER_NICK, buf ) && buf.Length() != 0) {
					aPost->iSender.iJabberNick().Append(buf);
				}
				if ( Settings().GetSettingL(SETTING_PHONENO, buf ) && buf.Length() != 0) {
					aPost->iSender.iPhoneNo().Append(buf);
				}
				TInt sett;
				if ( Settings().GetSettingL(SETTING_PUBLISH_SHARING, sett) ) {
					aPost->iSharing()=sett;
				}
				if ( Settings().GetSettingL(SETTING_PUBLISH_AUTOTAGS, sett) ) {
					aPost->iIncludedTagsBitField()=sett;
				}
				GetIMEI(aPost->iSender.iImei()); 
				// FIX ME: BT ADDR HERE
				iStorage.AddLocalL(aPost.get(), bm);

				TBool ok = iStorage.FirstL(CPostStorage::MediaPoolId(), iSort, iOrder, ETrue);
				if (ok) {
					TUint idx=iStorage.GetCurrentIndexL();
					iPostArray->AppendL(idx);
				}
			}
			iCurrentPostIndex=0;
			iPost=iStorage.GetByIndexL( this, (*iPostArray)[iCurrentPostIndex] );
		}
	}

// BEGIN: Graphic components-----------------------------------------------
	CreateWindowL();

	if (iStandAlone) {
		{
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("autotags1"));

			iAutoTagArray=CAutoTagArray::NewL(iAskForNames);
		}
		{
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("autotags2"));
			iAutoTags=CAutoTagListBox::NewL(iAutoTagArray, this, iTagIcons);
		}
		{
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("autotags3"));
			iAutoTags->ActivateL();
		}
		{
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("autotags4"));
			ShowAutoTags(EFalse);
		}
	}


	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("resources"));

		iLoadingBuf = iEikonEnv->AllocReadResourceL(R_LOADING);
		iDownloadingBuf = iEikonEnv->AllocReadResourceL(R_DOWNLOADING);
		iDefaultCommentBuf = iEikonEnv->AllocReadResourceL(R_DEFAULT_COMMENT);
		iDefaultSignatureBuf = iEikonEnv->AllocReadResourceL(R_DEFAULT_SIGNATURE);
		iDefaultTagsBuf = iEikonEnv->AllocReadResourceL(R_TAGS_DEFAULT);
	}
	
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("controls"));

		iSBFrame = new (ELeave) CEikScrollBarFrame(this, 0, ETrue);
		iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOn, CEikScrollBarFrame::EOn);
		iSBFrame->SetScrollBarManagement(CEikScrollBar::EVertical, CEikScrollBarFrame::EFloating);

		iLeftArrow = new (ELeave) CEikLabel;
		iLeftArrow->SetContainerWindowL(*this);
		iLeftArrow->SetFont(CEikonEnv::Static()->NormalFont());
		iLeftArrow->SetTextL(_L("<"));
		iLeftArrow->SetAlignment(TGulAlignment(EHLeftVCenter));
		iLeftArrow->OverrideColorL(EColorLabelText, KTextColor);
		iLeftArrow->SetPosition(TPoint(2,65));
		iLeftArrow->SetSize(iLeftArrow->MinimumSize());
		iLeftArrow->SetAllMarginsTo(0);
		iLeftArrow->SetFocusing(EFalse);

		iLeftArrowBg = new (ELeave) CEikLabel;
		iLeftArrowBg->SetContainerWindowL(*this);
		iLeftArrowBg->SetTextL(KNullDesC);
		iLeftArrowBg->SetExtent(TPoint(1,65), TSize(8,12));
		iLeftArrowBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iLeftArrowBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTagsBgColor);
		iLeftArrowBg->SetAllMarginsTo(0);
		iLeftArrowBg->SetFocusing(EFalse);
                
		iLeftArrowBlackBg = new (ELeave) CEikLabel;
		iLeftArrowBlackBg->SetContainerWindowL(*this);
		iLeftArrowBlackBg->SetTextL(KNullDesC);
		iLeftArrowBlackBg->SetExtent(TPoint(0,64), TSize(10,14));
		iLeftArrowBlackBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iLeftArrowBlackBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTextColor);
		iLeftArrowBlackBg->SetAllMarginsTo(0);
		iLeftArrowBlackBg->SetFocusing(EFalse);

		iRightArrow = new (ELeave) CEikLabel;
		iRightArrow->SetContainerWindowL(*this);
		iRightArrow->SetFont(CEikonEnv::Static()->NormalFont());
		iRightArrow->SetTextL(_L(">"));
		iRightArrow->SetAlignment(TGulAlignment(EHRightVCenter));
		iRightArrow->OverrideColorL(EColorLabelText, KTextColor);
		iRightArrow->SetPosition(TPoint(168,65));
		iRightArrow->SetSize(iRightArrow->MinimumSize());
		iRightArrow->SetAllMarginsTo(0);
		iRightArrow->SetFocusing(EFalse);

		iRightArrowBg = new (ELeave) CEikLabel;
		iRightArrowBg->SetContainerWindowL(*this);
		iRightArrowBg->SetTextL(KNullDesC);
		iRightArrowBg->SetExtent(TPoint(167,65), TSize(8,12));
		iRightArrowBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iRightArrowBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTagsBgColor);
		iRightArrowBg->SetAllMarginsTo(0);
		iRightArrowBg->SetFocusing(EFalse);

		iRightArrowBlackBg = new (ELeave) CEikLabel;
		iRightArrowBlackBg->SetContainerWindowL(*this);
		iRightArrowBlackBg->SetTextL(KNullDesC);
		iRightArrowBlackBg->SetExtent(TPoint(166,64), TSize(10,14));
		iRightArrowBlackBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iRightArrowBlackBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTextColor);
		iRightArrowBlackBg->SetAllMarginsTo(0);
		iRightArrowBlackBg->SetFocusing(EFalse);

		TInt time_left=120;
		if (iStandAlone) time_left=140;

		TInt aTimeHeight=15; TInt aTimeTop=125;
		iTimeLabel = new (ELeave) CEikLabel;
		iTimeLabel->SetContainerWindowL(*this);
		iTimeLabel->SetFont(LatinPlain12());
		iTimeLabel->SetAlignment(TGulAlignment(EHCenterVCenter));
		iTimeLabel->OverrideColorL(EColorLabelText, KTextColor);
		iTimeLabel->SetAllMarginsTo(0);
		iTimeLabel->SetPosition(TPoint(time_left, aTimeTop));
		iTimeLabel->SetSize(TSize(30, aTimeHeight));
		iTimeLabel->SetFocusing(EFalse);
		
		iTimeLabelBg = new (ELeave) CEikLabel;
		iTimeLabelBg->SetContainerWindowL(*this);
		iTimeLabelBg->SetTextL(KNullDesC);
		iTimeLabelBg->SetPosition(TPoint(time_left-1, aTimeTop-1));
		iTimeLabelBg->SetSize(TSize(32, aTimeHeight));
		iTimeLabelBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iTimeLabelBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTagsBgColor);
		iTimeLabelBg->SetAllMarginsTo(0);
		iTimeLabelBg->SetFocusing(EFalse);

		iTimeLabelBlackBg = new (ELeave) CEikLabel;
		iTimeLabelBlackBg->SetContainerWindowL(*this);
		iTimeLabelBlackBg->SetTextL(KNullDesC);
		iTimeLabelBlackBg->SetPosition(TPoint(time_left-2, aTimeTop-2));
		iTimeLabelBlackBg->SetSize(TSize(34, aTimeHeight+2));
		iTimeLabelBlackBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iTimeLabelBlackBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTextColor);
		iTimeLabelBlackBg->SetAllMarginsTo(0);
		iTimeLabelBlackBg->SetFocusing(EFalse);

		TInt count_left=73;
		TInt count_size=30;
		if (iStandAlone) {
			count_left=time_left-70;
			count_size=100;
		}
		iCountLabel = new (ELeave) CEikLabel;
		iCountLabel->SetContainerWindowL(*this);
		iCountLabel->SetFont(LatinPlain12());
		iCountLabel->SetAlignment(TGulAlignment(EHRightVCenter));
		iCountLabel->OverrideColorL(EColorLabelText, KTextColor);
		iCountLabel->SetAllMarginsTo(0);
		iCountLabel->SetPosition(TPoint(count_left, 2));
		iCountLabel->SetSize(TSize(count_size, 12));
		iCountLabel->SetFocusing(EFalse);
		
		iCountLabelBg = new (ELeave) CEikLabel;
		iCountLabelBg->SetContainerWindowL(*this);
		iCountLabelBg->SetTextL(KNullDesC);
		iCountLabelBg->SetPosition(TPoint(count_left-1, 1));
		iCountLabelBg->SetSize(TSize(count_size+2, 12));
		iCountLabelBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iCountLabelBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTagsBgColor);
		iCountLabelBg->SetAllMarginsTo(0);
		iCountLabelBg->SetFocusing(EFalse);

		iCountLabelBlackBg = new (ELeave) CEikLabel;
		iCountLabelBlackBg->SetContainerWindowL(*this);
		iCountLabelBlackBg->SetTextL(KNullDesC);
		iCountLabelBlackBg->SetPosition(TPoint(count_left-2, 0));
		iCountLabelBlackBg->SetSize(TSize(count_size+4, 14));
		iCountLabelBlackBg->SetEmphasis(CEikLabel::EFullEmphasis);
		iCountLabelBlackBg->OverrideColorL(EColorLabelHighlightFullEmphasis, KTextColor);
		iCountLabelBlackBg->SetAllMarginsTo(0);
		iCountLabelBlackBg->SetFocusing(EFalse);

		iLoadingLabel=new (ELeave) CEikLabel;
		iLoadingLabel->SetContainerWindowL( *this );
		iLoadingLabel->SetRect(KVideoRectangle);
		if (! iStandAlone) {
			iLoadingLabel->SetFont(LatinBold17());
			iLoadingLabel->SetAlignment(TGulAlignment(EHCenterVCenter));
		} else {
			iLoadingLabel->SetFont(LatinPlain12());
			iLoadingLabel->SetAlignment(TGulAlignment(EHLeftVTop));
		}
		iLoadingLabel->SetEmphasis(CEikLabel::EFullEmphasis);
		iLoadingLabel->OverrideColorL(EColorLabelText, KTextColor);
		iLoadingLabel->OverrideColorL(EColorLabelHighlightFullEmphasis, 
			KUnfocusedBgColor);
		iLoadingLabel->SetFocusing(EFalse);

		iErrorLabel=new (ELeave) CEikEdwin;
		iErrorLabel->ConstructL();
		iErrorLabel->SetContainerWindowL( *this );
		iErrorLabel->SetRect( TRect(TPoint(20, 30), TSize(136, 60)) );
		iErrorLabel->SetFocusing(EFalse);
		iErrorLabel->MakeVisible(EFalse);
		iErrorLabel->SetWordWrapL(ETrue);
		iErrorLabel->AddEdwinObserverL(this);
		iErrorLabel->ActivateL();

		iVolumeIndicator=new (ELeave) CEikLabel;
		iVolumeIndicator->SetContainerWindowL( *this );
		iVolumeIndicator->SetTextL(KNullDesC);
		iVolumeIndicator->SetFont(LatinPlain12());
		iVolumeIndicator->SetAlignment(TGulAlignment(EHCenterVCenter));
		iVolumeIndicator->OverrideColorL(EColorLabelText, KRgbBlue);
		iVolumeIndicator->SetFocusing(EFalse);
		iVolumeIndicator->SetExtent(TPoint(73,125), TSize(35, 20));
	}

	if (iView->Id()==KPostViewId) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("textcontrols1"));

		iComment = new (ELeave) OurOwnRichTextEditor(0);
		iComment->ConstructL(this, 3, 1100, 0);
		iComment->SetContainerWindowL(*this);

		if (!iStandAlone) {
			iComment->SetExtent(TPoint(1,141), TSize(174, 46));
		} else {
			iComment->SetExtent(TPoint(8,141), TSize(160, 46));
		}
		iComment->SetBackgroundColorL(KUnfocusedBgColor);
		iComment->SetReadOnly(ETrue);
		iComment->SetFocusing(EFalse);
	} else {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("textcontrols2"));

		iLoadingLabel->SetFocusing(ETrue);
		iControls->AppendL(iLoadingLabel);

		if (iAutoTags)
			iControls->AppendL(iAutoTags);

		iComment = new (ELeave) OurOwnRichTextEditor(iDefaultCommentBuf);
		iComment->SetAknEditorAllowedInputModes(EAknEditorAllInputModes);
		iComment->ConstructL(this, 2, 1000, EAknEditorFlagDefault);
		iComment->SetContainerWindowL(*this);

		if (! iStandAlone ) {
			iComment->SetExtent(TPoint(1,141), TSize(174, 29));
		} else {
			iComment->SetExtent(TPoint(8,141+16), TSize(160, 29));
		}
		iComment->SetBackgroundColorL(KUnfocusedBgColor);

		if (! iStandAlone ) {
			iSignature = new (ELeave) OurOwnRichTextEditor(iDefaultSignatureBuf);
			iSignature->SetAknEditorAllowedInputModes(EAknEditorAllInputModes);
			iSignature->ConstructL(this, 1, 50, 0);
			iSignature->SetContainerWindowL(*this);
			iSignature->SetExtent(TPoint(1,171), TSize(174, 16));
			iSignature->SetBackgroundColorL(KUnfocusedBgColor);
			iControls->AppendL(iComment);
			iControls->AppendL(iSignature);
		} else {
			iTags = new (ELeave) OurOwnRichTextEditor(iDefaultTagsBuf);
			iTags->SetAknEditorAllowedInputModes(EAknEditorAllInputModes);
			iTags->ConstructL(this, 1, 100, 0);
			iTags->SetContainerWindowL(*this);
			iTags->SetExtent(TPoint(8,141), TSize(160, 16));
			iTags->SetBackgroundColorL(KUnfocusedBgColor);
			iControls->AppendL(iTags);
			iControls->AppendL(iComment);
		}
	}

	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("selecttags"));
		iSelectTagLabel=new (ELeave) CEikLabel;
		iSelectTagLabel->SetContainerWindowL( *this );

		{
			TBuf<30> select_tags;
			iEikonEnv->ReadResourceAsDes16(select_tags, R_SELECT_TAGS);

			iSelectTagLabel->SetTextL(select_tags);
		}
		iSelectTagLabel->SetFont(iEikonEnv->NormalFont());
		iSelectTagLabel->SetAlignment(TGulAlignment(EHCenterVCenter));
		iSelectTagLabel->OverrideColorL(EColorLabelText, KRgbBlue);
		iSelectTagLabel->SetFocusing(EFalse);
		iSelectTagLabel->SetEmphasis(CEikLabel::EFullEmphasis);
		iSelectTagLabel->OverrideColorL(EColorLabelHighlightFullEmphasis, KUnfocusedBgColor);
		iSelectTagLabel->SetExtent(TPoint(0, 30), TSize(176, 14));
		iSelectTagLabel->MakeVisible(EFalse);
	}

	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("notify"));

		iNotifyControl=new (ELeave) CNotifyWindowControl;
		iNotifyControl->ConstructL(this);
		iNotifyControl->SetContainerWindowL(*this);
		if (!iStandAlone) 
			iNotifyControl->SetExtent(TPoint(176-19,0), TSize(9, 30));
		else
			iNotifyControl->SetExtent(TPoint(176-10, 15), TSize(9, 30));
		iControls->AppendL(iNotifyControl);
	}

// END: Graphic components-----------------------------------------------
	RestoreOriginalCBAL();

	SetRect(aRect);
	ActivateL();
	display_current();
}

void CContextMediaAppPostContainer::HandleEdwinEventL(CEikEdwin* /*aEdwin*/, TEdwinEvent /*aEventType*/) { }

CContextMediaAppPostContainer::~CContextMediaAppPostContainer()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("~CContextMediaAppPostContainer"));
	
	if (iLive) *iLive=0;

	delete iDisplayTimeOut;
	delete iVolumeDisplayTimeOut;
	delete iAutoTimer;
	if (!iLive) {
		delete iMediaDisplay;
		iMediaDisplay=0;
	}
	
	if (iVideoEngine) iVideoEngine->CloseController();
	if (iAudioEngine) iAudioEngine->CloseController();
	delete iAudioEngine;
	delete iVideoEngine;

	iStorage.Release(iPost, this); iPost=0;
	iStorage.Release(iParentPost, this); iParentPost=0;
	delete iPostArray;
	
	delete iLoadingLabel;
	delete iErrorLabel;
	delete iLeftArrow;
	delete iLeftArrowBg;
	delete iLeftArrowBlackBg;
	delete iRightArrow;
	delete iRightArrowBg;
	delete iRightArrowBlackBg;
	delete iTimeLabel;
	delete iTimeLabelBg;
	delete iTimeLabelBlackBg;
	delete iCountLabel;
	delete iCountLabelBg;
	delete iCountLabelBlackBg;
	delete iComment;
	delete iSignature;
	delete iTags;
	delete iVolumeIndicator;
	delete iSelectTagLabel;

	delete iControls;
	
	delete iLoadingBuf;
	delete iDownloadingBuf;
	delete iDefaultSignatureBuf;
	delete iDefaultCommentBuf;
	delete iDefaultTagsBuf;
	delete iNotifyControl;

	delete iAutoTags;
	delete iAutoTagArray;

	delete iSBFrame;
	delete iPresence;
}

void CContextMediaAppPostContainer::SetPresence(CBBPresence *aPresence)
{
	delete iPresence;
	iPresence=aPresence;
}
void CContextMediaAppPostContainer::SizeChanged()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("SizeChanged"));

	// no impl
}

TInt CContextMediaAppPostContainer::CountComponentControls() const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("CountComponentControls"));

	if (!iTags && !iSignature) return 20;
	return 21; 
}

void CContextMediaAppPostContainer::ShowAutoTags(TBool aDraw)
{
	if (!iAutoTags) return;
	TRect r( TPoint(15, 18), 
		TSize(iAutoTags->Width(), iAutoTags->RowHeight()*iAutoTagArray->MdcaCount()+
		iAutoTags->BorderWidth()*2));
	iAutoTags->SetRect(r);
	iAutoTags->MakeVisible(ETrue);
	iAutoTags->SetTopItemIndex(0);
	if (aDraw) iAutoTags->DrawNow();
	iAutoTagsIsVisible=ETrue;
}

void CContextMediaAppPostContainer::DrawMediaDisplay()
{
	if (iMediaDisplay && iMediaReady) {
		CWindowGc& gc = SystemGc();
		ActivateGc();
		if (iStandAlone) 
			gc.DrawBitmap(KPictureRectangleStandAlone, iMediaDisplay->Bitmap());
		else
			gc.DrawBitmap(KPictureRectangle, iMediaDisplay->Bitmap());
		DeactivateGc();
	} else {
		iLoadingLabel->DrawNow();
	}

	if (iErrorLabel->IsVisible()) iErrorLabel->DrawNow();

	iLeftArrowBlackBg->DrawNow();
	iLeftArrowBg->DrawNow();
	iLeftArrow->DrawNow();
	iRightArrowBlackBg->DrawNow();
	iRightArrowBg->DrawNow();
	iRightArrow->DrawNow();
	iCountLabelBlackBg->DrawNow();
	iCountLabelBg->DrawNow();
	iCountLabel->DrawNow();
	iTimeLabelBlackBg->DrawNow();
	iTimeLabelBg->DrawNow();
	iTimeLabel->DrawNow();
	iNotifyControl->DrawNow();

}

void CContextMediaAppPostContainer::AnimateAutoTags(TBool aIsVisible, TBool aWithBackground)
{
	if (!iAutoTags) return;
	if (iAutoTagsIsVisible==aIsVisible) return;
	iAutoTagsIsVisible=aIsVisible;

	CWindowGc& gc = SystemGc();

	TInt steps, h;
	h=iAutoTags->RowHeight();
	steps=iAutoTagArray->MdcaCount()-1;

	TRgb clearedbg=KRgbWhite;
	if (iLoadingLabel->IsFocused() && iView->Id()!=KPostViewId) clearedbg=KFocusedBgColor;

	if (aIsVisible) {
		ActivateGc();
		for (int i=0; i<steps; i++) {
			TRect r=iAutoTags->Rect();
			r.Move(0, -h);
			r.Resize(0, h);
			iAutoTags->SetRect(r);
			DeactivateGc();
			iAutoTags->SetTopItemIndex(iAutoTagArray->MdcaCount()-1-(i+1));
			iAutoTags->DrawNow();
			ActivateGc();
			if (i%2==1) iEikonEnv->Flush();
		}
		if (steps%2==0) iEikonEnv->Flush();
		DeactivateGc();
	} else {
		for (int i=steps; i>0; i--) {
			TRect r=iAutoTags->Rect();
			TRect cleared=r;
			cleared.Resize(0, -(r.Height()-h));
			r.Move(0, h);
			r.Resize(0, -h);
			iAutoTags->SetRect(r);
			iAutoTags->SetTopItemIndex(iAutoTagArray->MdcaCount()-1-(i-1));
			ActivateGc();
			if (!aWithBackground &&
					iMediaReady && iMediaDisplay && iMediaDisplay->IsVisible()) {
				TRect source=cleared;
				source.Move(-KPictureRectangleStandAlone.iTl.iX, -KPictureRectangleStandAlone.iTl.iY);
				gc.DrawBitmap(cleared, iMediaDisplay->Bitmap(), source);
			} else {
				gc.SetPenStyle(CGraphicsContext::ENullPen);
				gc.SetBrushColor(clearedbg);
				gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
				gc.DrawRect(cleared);
			}
			DeactivateGc();
			iAutoTags->DrawNow();
			if (i%2==0) iEikonEnv->Flush();
		}
		if (steps%2==1) iEikonEnv->Flush();
	}
}

CCoeControl* CContextMediaAppPostContainer::ComponentControl(TInt aIndex) const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("ComponentControl"));

#ifdef __WINS__
	TBuf<30> msg=_L("**CONTROL ");
	msg.AppendNum(aIndex);
	RDebug::Print(msg);
#endif
	switch ( aIndex ) {
		case 0: return iLoadingLabel;
		case 1: 
			if (!iLive && iMediaDisplay) return iMediaDisplay;
			else return iLoadingLabel;
		case 2: return iLeftArrowBlackBg;
		case 3: return iLeftArrowBg;
		case 4:	return iLeftArrow;
		case 5: return iRightArrowBlackBg;
		case 6: return iRightArrowBg;
		case 7: return iRightArrow;
		case 8:	return iTimeLabelBlackBg;
		case 9: return iTimeLabelBg;
		case 10: return iTimeLabel;
		case 11: return iCountLabelBlackBg;
		case 12: return iCountLabelBg;
		case 13: return iCountLabel;
		case 14: return iComment;
		case 15: return iNotifyControl;
		case 16: return iVolumeIndicator;
		case 17: return iSelectTagLabel;
		case 18: if (iAutoTags) {
				return iAutoTags;
			 }
			return iVolumeIndicator;
		case 19: return iErrorLabel;
		case 20: {
			if (iSignature) return iSignature;
			else return iTags;
			 }
	
		default: return 0;
        }
}

void CContextMediaAppPostContainer::Draw(const TRect& aRect) const
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("Draw"));

	if ( (iView->Id()==KPostViewId) && 
		( 
			(!iAutoTags && iCurrentScrollPos==0) || 
			(iAutoTags && iCurrentScrollPos==1)
			) ) {
		//necessary to display the comment properly. Reset the cursor position to beginning of text
		//makes it readable!
		iComment->MoveCursorL(TCursorPosition::EFPageUp, EFalse); 
	}

	CWindowGc& gc = SystemGc();
	gc.SetPenStyle(CGraphicsContext::ENullPen);
	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
		
	TRect re = iLoadingLabel->Rect();	
	gc.SetBrushColor(KRgbWhite);
	gc.DrawRect(re);

}

TKeyResponse CContextMediaAppPostContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("OfferKeyEventL"));

	//if (aType != EEventKey) return EKeyWasNotConsumed;

	/*if ( (aKeyEvent.iCode == EKeyCBA1) || (aKeyEvent.iCode == EKeyCBA2) ){
		if (iLoadingMedia) {
			return EKeyWasConsumed;
		} else {
			return EKeyWasNotConsumed;
		}
	}*/

	if (aType == EEventKey && aKeyEvent.iCode == EKeyYes){
		if (iView->Id()==KPostViewId) {
			if ((!iPlayingMedia)&& (!iLoadingMedia) && !iStandAlone) iView->HandleCommandL(EcontextmediaappCmdReply);
		} else {
			if (!iLoadingMedia) iView->HandleCommandL(EcontextmediaappCmdPublish);
		}
		return EKeyWasConsumed;
	}

	if (aType == EEventKey && aKeyEvent.iCode == EKeyUpArrow) {
		if (iPlayingMedia) {
			if (iMediaFileType==EAudio) {
				TInt vol = iAudioEngine->IncreaseVolume();
				ShowVolume(vol);
			} else if (iMediaFileType==EVideo) {
				TInt vol = iVideoEngine->IncreaseVolume();
				ShowVolume(vol);
			}
		} else {
			if (iView->Id()==KPostViewId) {
				if ( (!iAutoTags && iCurrentScrollPos>0) || iCurrentScrollPos>1) {
					iSBFrame->MoveThumbsBy(0, -1);
					iComment->MoveDisplayL(TCursorPosition::EFLineUp);
					iSBFrame->DrawScrollBarsNow();
					iCurrentScrollPos--;
				} else if (iAutoTags && iCurrentScrollPos==1) {
					iSBFrame->MoveThumbsBy(0, -1);
					AnimateAutoTags(ETrue);
					iSBFrame->DrawScrollBarsNow();
					iCurrentScrollPos--;
				}
			} else {
				if (iComment->IsFocused() ) {
					if (iComment->CursorPos() == 0) {			
						FocusPrevious();
					} else {
						iComment->MoveCursorL(TCursorPosition::EFLineUp, EFalse);
					}
				} else if (iTags && iTags->IsFocused()) {
					if (iTags->CursorPos() == 0) {			
						FocusPrevious();
					} else {
						iTags->MoveCursorL(TCursorPosition::EFLineUp, EFalse);
					}
				} else if (iSignature && iSignature->IsFocused()) {
					if (iSignature->CursorPos() == 0) {			
						FocusPrevious();
					} else {
						iSignature->MoveCursorL(TCursorPosition::EFLineUp, EFalse);
					}
				} else if (iAutoTags && iAutoTags->IsFocused()) {
					if (!iPost->iPresence() || iAutoTags->CurrentItemIndex() == iAutoTags->TopItemIndex()) {
						FocusPrevious();
					} else {
						iAutoTags->SetCurrentItemIndexAndDraw(iAutoTags->CurrentItemIndex()-1);
					}
				} else {
					FocusPrevious();
				}
			}
		}
		return EKeyWasConsumed;
	}

	if (aType == EEventKey && aKeyEvent.iCode == EKeyDownArrow) {
		if (iPlayingMedia) {
			if (iMediaFileType==EAudio) {
				TInt vol = iAudioEngine->DecreaseVolume();
				ShowVolume(vol);
			} else if (iMediaFileType==EVideo) {
				TInt vol = iVideoEngine->DecreaseVolume();
				ShowVolume(vol);
			}
		} else {
			if (iView->Id()==KPostViewId) {
				if (iAutoTags && iCurrentScrollPos==0) {
					iSBFrame->MoveThumbsBy(0, 1);
					AnimateAutoTags(EFalse);
					iSBFrame->DrawScrollBarsNow();
					iCurrentScrollPos++;
				} else if (iCurrentScrollPos+1<iMaxScrollPos) {
					iSBFrame->MoveThumbsBy(0, 1);
					iComment->MoveDisplayL(TCursorPosition::EFLineDown);
					iSBFrame->DrawScrollBarsNow();
					iCurrentScrollPos++;
				}
			} else {
				if (iComment->IsFocused() ) {
					if (iComment->CursorPos() == iComment->TextLength()) {			
						FocusNext();
					} else {
						iComment->MoveCursorL(TCursorPosition::EFLineDown, EFalse);
					}
				} else if (iTags && iTags->IsFocused()) {
					if (iTags->CursorPos() == iTags->TextLength()) {			
						FocusNext();
					} else {
						iTags->MoveCursorL(TCursorPosition::EFLineDown, EFalse);
					}
				} else if (iSignature && iSignature->IsFocused()) {
					if (iSignature->CursorPos() == iSignature->TextLength()) {			
						FocusNext();
					} else {
						iSignature->MoveCursorL(TCursorPosition::EFLineDown, EFalse);
					}
				} else if (iAutoTags && iAutoTags->IsFocused()) {
					if (iAutoTags->CurrentItemIndex() == iAutoTags->BottomItemIndex()) {
						FocusNext();
					} else {
						iAutoTags->SetCurrentItemIndexAndDraw(iAutoTags->CurrentItemIndex()+1);
					}
				} else {
					FocusNext();
				}
			}
			
		}
		return EKeyWasConsumed;
	}
	
	if (aType == EEventKey && aKeyEvent.iCode==EKeyLeftArrow) {
		if (iView->Id()==KPostViewId) {
                        if (!iLoadingMedia && !iPlayingMedia && has_previous()) {
				display_previous();
			}
		} else if (!iPlayingMedia) {
			if (iComment->IsFocused()) {
				return iComment->OfferKeyEventL(aKeyEvent, aType);
			} else if (iTags && iTags->IsFocused()) {
				return iTags->OfferKeyEventL(aKeyEvent, aType);
			} else if (iSignature && iSignature->IsFocused()) {
				return iSignature->OfferKeyEventL(aKeyEvent, aType);
			} else if (iMediaBrowsingOn && !iLoadingMedia && !iPlayingMedia && has_previous()) {
				display_previous();
			}
		}
		return EKeyWasConsumed;
	}

	if (aType == EEventKey && aKeyEvent.iCode==EKeyRightArrow) {
		if (iView->Id()==KPostViewId) {
			if (!iLoadingMedia && !iPlayingMedia && has_next()) display_next();
		} else if (!iPlayingMedia){
			if (iComment->IsFocused()) {
				return iComment->OfferKeyEventL(aKeyEvent, aType);
			} else if (iTags && iTags->IsFocused()) {
				return iTags->OfferKeyEventL(aKeyEvent, aType);
			} else if (iSignature && iSignature->IsFocused()) {
				return iSignature->OfferKeyEventL(aKeyEvent, aType);
			} else if (iMediaBrowsingOn && !iLoadingMedia && !iPlayingMedia && has_next()) {
				display_next();
			}
		}
		return EKeyWasConsumed;
	}

	if (	(iComment && iComment->IsFocused()) ||
		(iTags && iTags->IsFocused()) ||
		(iSignature && iSignature->IsFocused()) ) {
	} else {
		if ( aType == EEventKey && aKeyEvent.iCode == EKeyBackspace) {
			if (iPlayingMedia) {
				return EKeyWasConsumed;
			} else if (iAutoTags && iAutoTags->IsFocused() ) {
				return iAutoTags->OfferKeyEventL(aKeyEvent, aType);
			} else 	if (iView->Id()==KPostViewId) {
				return EKeyWasNotConsumed;
			} else {
				if (iLoadingLabel->IsFocused()){
					iView->HandleCommandL(EcontextmediaappCmdDelete);
					return EKeyWasConsumed;
				} else {
					return EKeyWasNotConsumed;
				}
			}
		}
	}

	if ( aType == EEventKey && (aKeyEvent.iCode == EKeyOK) && iTags && iTags->IsFocused() && iTagStorage) {
		TInt added_from;
		TCursorSelection sel=iTags->Selection();
		// if the selection extends beyond the new text length we get
		// an ETEXT-12 panic, so reset selection
		iTags->ClearSelectionL();
		iSelectTagLabel->MakeVisible(ETrue);
		iSelectTagLabel->DrawNow();
		if ( SelectTagsFromListL(*iTagStorage, iTags->RichText(), added_from,
				sel.LowerPos(), sel.HigherPos() ) ) {
			iSelectTagLabel->MakeVisible(EFalse);
			iTags->HandleTextChangedL();
			iTags->SetCursorPosL(added_from, EFalse);
			iTags->SetCursorPosL(iTags->TextLength(), ETrue);
			DrawMediaDisplay();
		} else {
			iSelectTagLabel->MakeVisible(EFalse);
			if (sel.iCursorPos<iTags->TextLength() && sel.iAnchorPos<iTags->TextLength() ) {
				// with FEPs clearing the selection may delete text,
				// so we must be careful not to try to select text that is
				// not there
				iTags->SetSelectionL(sel.iCursorPos, sel.iAnchorPos);
			}
			DrawMediaDisplay();
		}
		return EKeyWasConsumed;
	}
	if ( aType == EEventKey && (aKeyEvent.iCode == EKeyOK || aKeyEvent.iCode==EKeyDelete) && iAutoTags && iAutoTags->IsFocused() ){
		return iAutoTags->OfferKeyEventL(aKeyEvent, aType);
	}
	if ( aType == EEventKey && (aKeyEvent.iCode == EKeyOK) && IsCurrentPlayableMedia()){
		if (iView->Id()==KPostViewId) {
			if (!iPlayingMedia) {
				Play();
			} else {
				if (iMediaFileType==EVideo) {
					if ( iVideoEngine->GetEngineState() == CVideoEngine::EPPlaying ) {
						Pause();
					} else if (iVideoEngine->GetEngineState() == CVideoEngine::EPPaused) {
						Resume();
					} 
				} else if (iMediaFileType==EAudio){
					if ( iAudioEngine->GetEngineState() == CAudioEngine::EPPlaying ) {
						Pause();
					} else if (iAudioEngine->GetEngineState() == CAudioEngine::EPPaused) {
						Resume();
					} 
				}
			}
		} else {
			if ( iLoadingLabel->IsFocused() ) {
				if (!iPlayingMedia) {
					Play();
				} else {
					if (iMediaFileType==EVideo) {
						if ( iVideoEngine->GetEngineState() == CVideoEngine::EPPlaying ) {
							Pause();
						} else if (iVideoEngine->GetEngineState() == CVideoEngine::EPPaused) {
							Resume();
						} 
					} else if (iMediaFileType==EAudio){
						if ( iAudioEngine->GetEngineState() == CAudioEngine::EPPlaying ) {
							Pause();
						} else if (iAudioEngine->GetEngineState() == CAudioEngine::EPPaused) {
							Resume();
						} 
					}
				}
			}

		}
		return EKeyWasConsumed;
	}

	if (iComment && iComment->IsFocused()) return iComment->OfferKeyEventL(aKeyEvent, aType);
	if (iTags && iTags->IsFocused()) return iTags->OfferKeyEventL(aKeyEvent, aType);
	if (iSignature && iSignature->IsFocused()) return iSignature->OfferKeyEventL(aKeyEvent, aType);
	return EKeyWasNotConsumed;
}

void CContextMediaAppPostContainer::PostEvent(CCMPost* aParent, CCMPost* aChild, TEvent aEvent)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("PostEvent"));

	TBool count_changed=EFalse;
	if (iCurrentPostIndex!=-1 && aChild->iLocalDatabaseId()==(*iPostArray)[iCurrentPostIndex]) {
		if ( (aEvent == EMediaLoaded) || (aEvent == EErrorUpdated) ) {
			if (DisplayPicture())
				DrawNow();
		}
		if (aEvent == EPostDeleted || aEvent == EParentChanged ) {
			if (has_previous()) iCurrentPostIndex--;
			else if (has_next()) iCurrentPostIndex++;
			else iCurrentPostIndex=-1;

			iCurrentChanged=ETrue;
		}
	} 
	if (aParent==iParentPost) {
		if (aEvent == EPostDeleted || aEvent == EParentChanged ) {
			int delete_from=-1;
			int count=iPostArray->Count();
			for (int i=0; i<count; i++) {
				if ( (*iPostArray)[i]==aChild->iLocalDatabaseId() ) {
					delete_from=i;
					break;
				}
			}
			if (delete_from!=-1) {
				count_changed=ETrue;
				for (int i=delete_from; i<count-1; i++) {
					(*iPostArray)[i]=(*iPostArray)[i+1];
				}
				iPostArray->Delete(count-1);
				if (iCurrentPostIndex>=delete_from) {
					--iCurrentPostIndex;
					iCurrentChanged=ETrue;
				}
			}
		}
	}
	if (iCurrentChanged) {
		iDisplayTimeOut->Wait(0);
	} else if (count_changed) {
		CC_TRAPD(err, SetCountTextL());
		iCountLabel->DrawDeferred();
	}
}

TBool CContextMediaAppPostContainer::has_next()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("has_next"));

	return (iPostArray->Count() > iCurrentPostIndex+1);
}

TBool CContextMediaAppPostContainer::has_previous()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("has_previous"));

	return (iCurrentPostIndex>=1);	
}

void CContextMediaAppPostContainer::display_next()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_next"));

	if (iLive) *iLive=2;

	if (iView->Id()!=KPostViewId) {
		save_contribution_to_post(EFalse);
		iStorage.UpdatePostL(iPost); // save post to storage
	}

	iCurrentPostIndex++;

	iStorage.Release(iPost, this); iPost=0;
	CC_TRAP(iLoadError, iPost=iStorage.GetByIndexL( this, (*iPostArray)[iCurrentPostIndex] ));
	display_current();
	DrawNow();
}

void CContextMediaAppPostContainer::display_previous()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_previous"));

	if (iLive) *iLive=2;

	if (iView->Id()!=KPostViewId) {
		save_contribution_to_post(EFalse);
		TRAPD(ignore, iStorage.UpdatePostL(iPost)); // save post to storage
	}
	iCurrentPostIndex--;

	iStorage.Release(iPost, this); iPost=0;
	CC_TRAP(iLoadError, iPost=iStorage.GetByIndexL(this, (*iPostArray)[iCurrentPostIndex]));
	display_current();
	DrawNow();
}

void OurOwnRichTextEditor::Zero()
{
	RichText()->Reset();
}

void OurOwnRichTextEditor::AppendParagraph(const TDesC& aText, TRgb aColor, CParaFormat::TAlignment aAlignment)
{
	if (aText.Length()==0) return;

	CParaFormat paraFormat;
	TParaFormatMask paraFormatMask;
	TFontSpec fontspec = LatinPlain12()->FontSpecInTwips();
	TCharFormat charFormat( fontspec.iTypeface.iName, fontspec.iHeight );
	TCharFormatMask charFormatMask;

	TInt pos=RichText()->DocumentLength();
	if (pos>0) {
		RichText()->InsertL(pos, CEditableText::EParagraphDelimiter);
		pos=RichText()->DocumentLength();
	}
	RichText()->InsertL(pos, aText);
	RichText()->GetParaFormatL(&paraFormat, paraFormatMask, pos, aText.Length());
	paraFormatMask.SetAttrib(EAttLineSpacingControl);
	paraFormatMask.SetAttrib(EAttLineSpacing);
	paraFormatMask.SetAttrib(EAttAlignment);
	paraFormat.iLineSpacingControl = CParaFormat::ELineSpacingExactlyInTwips;
	paraFormat.iLineSpacingInTwips = 115;
	paraFormat.iHorizontalAlignment = aAlignment;
	RichText()->ApplyParaFormatL(&paraFormat, paraFormatMask, pos, aText.Length());
	charFormat.iFontPresentation.iTextColor=aColor;
	charFormatMask.SetAttrib(EAttColor);
	charFormatMask.SetAttrib(EAttFontTypeface);
	charFormatMask.SetAttrib(EAttFontHeight);
	RichText()->ApplyCharFormatL(charFormat, charFormatMask, pos, aText.Length());
}

void CContextMediaAppPostContainer::SetCountTextL()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("SetCountTextL"));
	TBuf<40> count;
	if (iStandAlone) {
		if (iNode == iStorage.RootId()) {
			refcounted_ptr<CCMPost> root(iStorage.GetRootL(0));
			count.Append(root->iBodyText->Value().Left(20));
			count.Append(_L(":"));
		}
		if (iParentPost) {
			count.Append(iParentPost->iBodyText->Value().Left(20));
		}
		count.Append(_L(" "));
	}
	count.AppendFormat(_L("%d/%d"), iCurrentPostIndex+1, iPostArray->Count());
	iCountLabel->SetTextL(count);
}

void CContextMediaAppPostContainer::display_current(TInt aError)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current"));

	bool do_draw=false;
	if (iCurrentChanged) {
		if (iCurrentPostIndex<0) {
			iView->HandleCommandL(EAknSoftkeyBack);
			return;
		} else {
			iStorage.Release(iPost, this); iPost=0;
			CC_TRAP(iLoadError, iPost=iStorage.GetByIndexL(this, (*iPostArray)[iCurrentPostIndex]));
			do_draw=true;
		}
	}
	iErrorLabel->MakeVisible(EFalse);
	iCurrentChanged=EFalse;
	if (iLoadError!=KErrNone) {
		iErrorMessage=_L("Error loading data: ");
		iErrorMessage.AppendNum(iLoadError);
		iErrorLabel->SetTextL(&iErrorMessage);
		iErrorLabel->MakeVisible(ETrue);
		iLoadingLabel->SetTextL(iErrorMessage);
		do_draw=ETrue;
	}
	/* get current post */
	CCMPost * aPost = iPost;
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_reset"));
		/* reset visual components */
		if (iMediaDisplay) iMediaDisplay->MakeVisible(EFalse);
	}
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_2"));
		if (aError==KErrNone) {
			if (iLoadError==KErrNone)
				iLoadingLabel->SetTextL(_L("Loading..."));
		} else if (aError==KErrNotFound) {
			iLoadingLabel->SetTextL(_L("Media not found"));
		} else {
			TBuf<30> msg=_L("Error getting media ");
			msg.AppendNum(aError);
			iLoadingLabel->SetTextL(msg);
		}
	}


	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_1"));
		if ( iPost && (iNode == CPostStorage::RootId() || iParentPost==0)) {
			iStorage.Release(iParentPost, this); iParentPost=0;
			CC_TRAPD(err, iParentPost=iStorage.GetByPostIdL( this, iPost->iParentId() ));
		} 
	}

	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_3"));
		if (iAutoTagArray) iAutoTagArray->SetPost(iPost);
		iMediaReady=EFalse;
		iGoneUpFromAutoTags=EFalse;

		SetCountTextL();

		iMediaReady = EFalse;
		iMediaFileType = EUnknown;
		iRightArrow->MakeVisible(has_next());
		iRightArrowBg->MakeVisible(has_next());
		iRightArrowBlackBg->MakeVisible(has_next());
		iLeftArrow->MakeVisible(has_previous());
		iLeftArrowBg->MakeVisible(has_previous());
		iLeftArrowBlackBg->MakeVisible(has_previous());
	}
	TRgb KRgbTags(0, 100, 0);
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_4"));

		if (!iStandAlone && iView->Id()==KPostViewId) {
			/* set previous view indices to reflect browsing movement */
			CAknViewAppUi * appui = (CAknViewAppUi *)(iEikonEnv->AppUi());
			CContextMediaAppThreadView * view = (CContextMediaAppThreadView *)(appui->View(KThreadViewId));
			if (view) {
				view->SetCurrentIndex(get_current_idx()+1);
				view->SetTopIndex(get_top_idx()+1);
			}
		}
	}

	/* update text */
	TInt lines=0;
	if (aPost) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_5"));
		TBuf<5> time;
		TTime now=GetTime();
		TTime timestamp=ContextToLocal(aPost->iTimeStamp());
		if ((now.DateTime().Year()==timestamp.DateTime().Year()) 
			&& (now.DateTime().Month()==timestamp.DateTime().Month())
			&& (now.DateTime().Day()==timestamp.DateTime().Day()) ) {
			
			timestamp.FormatL(time, _L("%F%H:%T"));
		} else {
			//timestamp.FormatL(time, _L("%F%D/%M"));
			timestamp.FormatL(time, _L("%F%M-%D"));
		}
		iTimeLabel->SetTextL(time);

		if (aPost->iErrorInfo) {
			iErrorLabel->SetTextL( &(aPost->iErrorInfo->UserMessage()) );
			iErrorLabel->MakeVisible(ETrue);
		}

		if (iView->Id()==KPostViewId) {
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_5_1"));
			iComment->Zero();
			iComment->AppendParagraph(aPost->iTag->Value(), KRgbTags, CParaFormat::EJustifiedAlign);
			iComment->AppendParagraph(aPost->iBodyText->Value(), KRgbBlack, CParaFormat::EJustifiedAlign);
			if (!iStandAlone) {
				iComment->AppendParagraph(aPost->iSender.iName(), 
					KRgbBlue, CParaFormat::ERightAlign);
			}
		} else {
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_5_2"));
             		if (aPost->iBodyText->Value().Length()) {
				iComment->Zero();
				iComment->AppendParagraph(aPost->iBodyText->Value(), KRgbBlack, 
					CParaFormat::EJustifiedAlign);
			} else {
				iComment->Reset();
			}

			if (! iStandAlone ) {
				if (aPost->iSender.iName().Length()) {
					iSignature->Zero();
					iSignature->AppendParagraph(aPost->iSender.iName(), 
						KRgbBlack, CParaFormat::EJustifiedAlign);
				} else {
					iSignature->Reset();
				}
			} else {
				if (aPost->iTag->Value().Length()) {
					iTags->Zero();
					iTags->AppendParagraph(aPost->iTag->Value(), 
						KRgbBlack, CParaFormat::EJustifiedAlign);
				} else {
					iTags->Reset();
				}
			}
		}
		iComment->HandleTextChangedL();
		if (iSignature) iSignature->HandleTextChangedL();
		if (iTags) iTags->HandleTextChangedL();
	} else {
		iTimeLabel->SetTextL(KNullDesC);
		if (iTags) {
			iTags->Reset();
			iTags->HandleTextChangedL();
		}
		if (iSignature) {
			iSignature->Reset();
			iSignature->HandleTextChangedL();
		}
		iComment->Zero();
		iComment->AppendParagraph(_L(" "), KRgbTags, CParaFormat::ELeftAlign);
		iComment->HandleTextChangedL();
	}
	
	// adjust up&down arrow indicators to scroll text, if necessary
	if (iView->Id()==KPostViewId) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_6"));
		/* little hack to know how many scrolling positions we should have for the scroll bars */
		iComment->MoveCursorL(TCursorPosition::EFPageUp, EFalse);
		while (iComment->CursorPos()!=iComment->TextLength()) {
			iComment->MoveCursorL(TCursorPosition::EFLineDown, EFalse);
			lines++;
		}
		
		// Handling of the textbox+up/down arrow below
		iCurrentScrollPos=0;
		iMaxScrollPos=lines-3; // 3lines displayed on screen
		if (iAutoTags) {
			iCurrentScrollPos=1;
			if (iMaxScrollPos<=0) iMaxScrollPos=1;
			else iMaxScrollPos++;
		}
		if (iMaxScrollPos>0) {
			iMaxScrollPos++;
			iModel = TEikScrollBarModel(iMaxScrollPos, 0, 0);
			iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff,CEikScrollBarFrame::EAuto);
			iSBFrame->Tile(&iModel);
			if (iCurrentScrollPos) iSBFrame->MoveThumbsBy(0, iCurrentScrollPos);
		} else {
			iModel = TEikScrollBarModel(0, 0, 0);
			iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff,CEikScrollBarFrame::EOff);
			iSBFrame->Tile(&iModel);
		}
		iSBFrame->DrawScrollBarsNow();
	}

	// set default focus
	if (iView->Id()==KPostViewId) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_7_1"));
		iLoadingLabel->SetFocus(ETrue);
	} else {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_7_2"));
		int i=0;
		for(i=0; i<iControls->Count();i++) {
			CCoeControl* cntrl=iControls->At(i);
			cntrl->SetFocus(EFalse);
		}
	
		if (!iMediaBrowsingOn) {
			iComment->PrepareForFocusGainL();
			iComment->SetFocus(ETrue);
			iComment->OverrideColorL(EColorControlBackground, KFocusedBgColor);
			iComment->OverrideColorL(EColorLabelHighlightFullEmphasis, KFocusedBgColor);
		} else {
			iLoadingLabel->SetFocus(ETrue);
			iLoadingLabel->OverrideColorL(EColorControlBackground, KFocusedBgColor);
			iLoadingLabel->OverrideColorL(EColorLabelHighlightFullEmphasis, KFocusedBgColor);
		}
	}

	// make the display asynchronous but with no delay
	if (iPost) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("display_current_8"));
		if (!iMediaBrowsingOn) iDisplayTimeOut->Wait(1);
		else iDisplayTimeOut->Wait(0);
		ShowAutoTags(EFalse);
	} else {
		if (iAutoTags) iAutoTags->MakeVisible(EFalse);
	}
	if (do_draw) DrawNow();
}

TBool CContextMediaAppPostContainer::CloseCurrentMedia()
{
	if (iLive) return EFalse;
	delete iMediaDisplay; iMediaDisplay=0;
	CC_TRAPD(err, iLoadingLabel->SetTextL(_L("Closed for sending")));
	return ETrue;
}

TBool CContextMediaAppPostContainer::DisplayPicture()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPicture"));

	TInt err;
	CC_TRAP(err, iLoadingLabel->SetTextL(_L("Loading....")));
	if (iLive) {
		iWaitCount++;
		TBuf<40> msg=_L("Loading...");
		msg.AppendNum(iWaitCount);
		if (!iAutoTags) CC_TRAP(err, iLoadingLabel->SetTextL(msg));
		DrawNow();
		if (iWaitCount>1)
			iDisplayTimeOut->Wait(1);
		else
			iDisplayTimeOut->Wait(0);
		return EFalse;
	}
	iWaitCount=0;
	if (!iAutoTags) DrawNow();

	TInt aLive=1; iLive=&aLive;

	CPAlbImageViewerBasic* localMediaDisplay=iMediaDisplay;
	iMediaDisplay=0;
	CC_TRAP(err, DisplayPictureInnerL(aLive, localMediaDisplay));
	if (err!=KErrNone && aLive==1) {
		iMediaDisplay=localMediaDisplay;
		if (iMediaDisplay) iMediaDisplay->MakeVisible(EFalse);
		TBuf<50> msg;
		TBool retry=EFalse;
		if (err!=KErrNotFound && err!=KErrPathNotFound) {
			msg=_L("Error displaying:\n");
			msg.AppendNum(err);
		} else {
			msg=_L("Media not\nfound");
		}
		if (err==KErrInUse) retry=ETrue;
		CC_TRAP(err, iLoadingLabel->SetTextL(msg));
		if (iAutoTags && !iAutoTags->IsFocused()) AnimateAutoTags(EFalse, ETrue);
		DrawDeferred();
		if (retry) iDisplayTimeOut->Wait(1);
	}

	TBool ret=ETrue;
	if (aLive!=1) {
		ret=EFalse;
		delete localMediaDisplay; localMediaDisplay=0;
		iMediaDisplay=0;
	} else {
		iMediaDisplay=localMediaDisplay;
	}
	iLive=0;

	if (err!=KErrNone) return EFalse;;
	return ret;
}

void CContextMediaAppPostContainer::DisplayPictureInnerL(TInt& aLive,
							 CPAlbImageViewerBasic* & localMediaDisplay)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL"));


	if (!iPost) return;

	//delete localMediaDisplay; localMediaDisplay=0;

	iMediaReady=EFalse;
	if (!localMediaDisplay) {
#if 1
		if (!iStandAlone) { 
			localMediaDisplay = CPAlbImageViewerBasic::NewL(this, KPictureRectangle);
		} else {
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_1"));
			localMediaDisplay = CPAlbImageViewerBasic::NewL(this, KPictureRectangleStandAlone);
		}
#else
		localMediaDisplay = CPAlbImageViewerBasic::NewL(this, KPictureRectangle);
#endif
	} else {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2"));
		localMediaDisplay->MakeVisible(EFalse);
		DrawNow();
	}

	//iLoadingMedia=ETrue;

	const CCMPost * aPost = iPost;
	TBuf<255> msg;

	if (iStandAlone) iLoadingLabel->SetTextL(_L(""));
	if (!aPost) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_1"));
		if (iView->Id()==KPostViewId) {
			msg.Append(_L("err: no post!\n")); //FIX ME: use resource
			iLoadingLabel->SetTextL(msg);
			iLoadingMedia=EFalse;
		} else {
			msg.Append(_L("No media found!\n")); // FIX ME: use resource
			iLoadingLabel->SetTextL(msg);
			iLoadingMedia=EFalse;
			CEikButtonGroupContainer * cba = CEikButtonGroupContainer::Current();
			if( cba) {
				cba->SetCommandSetL( R_AVKON_SOFTKEYS_BACK );
				cba->DrawNow();
			}
		}
		return;
	} else if (aPost->iErrorCode()!=0) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_2"));
		msg.AppendFormat(_L("err %d:\n"), aPost->iErrorCode());
		msg.Append(aPost->iErrorDescr());
		iLoadingLabel->SetTextL(msg);
	} else if (aPost->iMediaFileName().Length()==0) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3"));
		msg.Append(*iDownloadingBuf);
		iLoadingLabel->SetTextL(msg);	
		iCMNetwork.FetchMedia(aPost);
	} else if (aPost->iMediaFileName().Length()!=0) {
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_4"));
		if (!iAutoTags) {
			iLoadingLabel->SetTextL(*iLoadingBuf);
			DrawNow();
		}
		TFileName fn;
		if (aPost->iContentType().FindF(_L("image/"))!=KErrNotFound) {
			fn = aPost->iMediaFileName();
			iMediaFileType = EImage;
		} else if (aPost->iContentType().FindF(_L("video/"))!=KErrNotFound){
			GetVideoMbmPath(fn);
			iMediaFileType = EVideo;
		} else if (aPost->iContentType().FindF(_L("audio/"))!=KErrNotFound){
			GetAudioMbmPath(fn);
			iMediaFileType = EAudio;
		} else {
			GetUnknownMbmPath(fn);
			iMediaFileType = EUnknown;
		}

		TInt err=KErrNone;
		if (!localMediaDisplay->IsBusy()) {
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3"));
#ifdef __S60V2__
			localMediaDisplay->ReleaseImageLoader();
#endif
			localMediaDisplay->ReleaseMemoryAndVisibleBitmap();
			RDebug::Print(_L("Release mem, about to load"));
#ifdef __S60V2__
			// !Efficient loading of large bitmaps!
			localMediaDisplay->SetImageNameAndDisplaymodeL(fn, EColor64K);
			if (aLive==0) {
				CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3_1"));
				delete localMediaDisplay; localMediaDisplay=0;
				return;
			}
			if (aLive==2) return;
			if (iStandAlone) {
				CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3_2"));
				while (localMediaDisplay->ZoomRatio()>-4) {
					CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3_3"));
					localMediaDisplay->ZoomOutL();
					if (aLive==0) {
						delete localMediaDisplay; localMediaDisplay=0;
					}
					if (aLive==0 || aLive==2) return;
				}
			}
			CC_TRAPD(errd, localMediaDisplay->LoadImageL());
#else 
			CC_TRAPD(errd, err=localMediaDisplay->LoadImageL(fn, EColor4K));
#endif
			if (aLive==0) {
				delete localMediaDisplay; localMediaDisplay=0;
				return;
			}
			if (aLive==2) return;
			if (errd) err=errd;
			if (err==KErrNone) {
				CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3_4"));
#if 1
				if (!iStandAlone)
					localMediaDisplay->FreeScaleL(KPictureSize, ETrue);
				else
					localMediaDisplay->FreeScaleL(KPictureSizeStandAlone, ETrue);
#else
				localMediaDisplay->FreeScaleL(KPictureSize, EFalse);
#endif
			}
			if (aLive==0) {
				delete localMediaDisplay; localMediaDisplay=0;
				return;
			}
			if (aLive==2) return;
			{
				CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_3_5"));
#ifdef __S60V2__
				localMediaDisplay->ReleaseImageLoader();
#endif
			}
		} else {
			CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_2_4"));
			iDisplayTimeOut->Wait(1);
		}

		if (err!=KErrNone) {
			msg.AppendFormat(_L("Err %d:\n can't open\n"), err);
			msg.Append(fn);
			iLoadingLabel->SetTextL(msg);
		} else {
			if (!iStandAlone && iView->Id()==KPostViewId) {
				mark_as_read();
			}
			iMediaReady=ETrue;
			iLoadingLabel->SetTextL(KNullDesC);
		}
	}
	{
		CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("DisplayPictureL_3"));
		iLoadingMedia=EFalse;
		if (iAutoTags && !iAutoTags->IsFocused()) AnimateAutoTags(EFalse, ETrue);
		localMediaDisplay->MakeVisible(iMediaReady);
		if (iLoadingLabel->Text()->Length()>1) iLoadingLabel->MakeVisible(ETrue);
		DrawDeferred();
	}
}

void CContextMediaAppPostContainer::expired(CBase * source)
{
	if (source==iDisplayTimeOut) {
		if (!iCurrentChanged) {
			DisplayPicture();
		} else {
			display_current();
		}
	} else if (source==iVolumeDisplayTimeOut) {
		iVolumeIndicator->SetTextL(KNullDesC);
		iVolumeIndicator->DrawDeferred();
	} else if (source==iAutoTimer) {
		if (iAutoTags->IsFocused()) return;
		AnimateAutoTags(EFalse);
	}
}

EXPORT_C void CContextMediaAppPostContainer::Play()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("Play"));

	if (!iMediaReady) return;
	if (!IsCurrentPlayableMedia()) return;
	CCMPost * aPost = iPost;

	CC_TRAPD(ignore, PlayMediaFileL(aPost->iMediaFileName());)
}

void CContextMediaAppPostContainer::PlayMediaFileL(const TDesC& aFilename)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("PlayMediaFileL"));

	if (!IsCurrentPlayableMedia()) return;
	iPlayingMedia = ETrue;
	iRightArrow->MakeVisible(EFalse);
	iRightArrowBlackBg->MakeVisible(EFalse);
	iRightArrowBg->MakeVisible(EFalse);
	iLeftArrow->MakeVisible(EFalse);
	iLeftArrowBlackBg->MakeVisible(EFalse);
	iLeftArrowBg->MakeVisible(EFalse);
	iTimeLabel->MakeVisible(EFalse);
	iTimeLabelBg->MakeVisible(EFalse);
	iTimeLabelBlackBg->MakeVisible(EFalse);
	iCountLabel->MakeVisible(EFalse);
	iCountLabelBg->MakeVisible(EFalse);
	iCountLabelBlackBg->MakeVisible(EFalse);

	TInt volume=0;
	Settings().GetSettingL(SETTING_PLAYBACK_VOLUME, volume);
	if (iMediaFileType==EVideo) {
		iMediaReady=EFalse;
		iMediaDisplay->MakeVisible(EFalse);
		iLoadingLabel->SetTextL(*iLoadingBuf);

		iMediaDisplay->DrawNow();
		iLoadingLabel->DrawNow();
		
		TRect r = KVideoRectangle2;
		iVideoEngine->SetNewFileL( aFilename );
		iVideoEngine->InitControllerL(this, ControlEnv()->WsSession(), 
						*(ControlEnv()->ScreenDevice()),
						Window(), r, r, volume);
	} else if (iMediaFileType==EAudio) {
		iAudioEngine->SetNewFileL(aFilename);
		iAudioEngine->InitControllerL(this, volume);
	} 
}

void CContextMediaAppPostContainer::PauseMediaFileL()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("PauseMediaFileL"));

	if (iMediaFileType==EVideo) {
		iVideoEngine->DoPauseL();
	} else if (iMediaFileType==EAudio){
		iAudioEngine->DoPauseL();
	}

	CEikButtonGroupContainer * cba = CEikButtonGroupContainer::Current();
	if( cba) {
		cba->SetCommandSetL( R_CONTINUE_STOP );
		cba->DrawNow();
	}
}

void CContextMediaAppPostContainer::StopMediaFileL()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("StopMediaFileL"));

	if (iMediaFileType==EVideo) {
		iVideoEngine->DoStopL();
	} else if (iMediaFileType==EAudio){
		iAudioEngine->DoStopL();
	}
}	

void CContextMediaAppPostContainer::ResumeMediaFileL()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("ResumeMediaFileL"));

	if (iMediaFileType==EVideo) {
		if ( iVideoEngine->GetEngineState() != CVideoEngine::EPPaused ) return;
		iVideoEngine->DoPlayL();
	} else if (iMediaFileType==EAudio){
		if ( iAudioEngine->GetEngineState() != CAudioEngine::EPPaused ) return;
		iAudioEngine->DoPlayL();
	}
	
	CEikButtonGroupContainer * cba = CEikButtonGroupContainer::Current();
	if( cba) {
		cba->SetCommandSetL( R_PAUSE_STOP );
		cba->DrawNow();
	}		
}

void CContextMediaAppPostContainer::PlayCompletedL(TInt aError)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("PlayCompletedL"));

	iRightArrow->MakeVisible(has_next());
	iRightArrowBg->MakeVisible(has_next());
	iRightArrowBlackBg->MakeVisible(has_next());
	iLeftArrow->MakeVisible(has_previous());
	iLeftArrowBg->MakeVisible(has_previous());
	iLeftArrowBlackBg->MakeVisible(has_previous());
	iTimeLabel->MakeVisible(ETrue);
	iTimeLabelBg->MakeVisible(ETrue);
	iTimeLabelBlackBg->MakeVisible(ETrue);
	iCountLabel->MakeVisible(ETrue);
	iCountLabelBg->MakeVisible(ETrue);
	iCountLabelBlackBg->MakeVisible(ETrue);
	iLoadingLabel->MakeVisible(ETrue);

	RestoreOriginalCBAL();
	iPlayingMedia=EFalse;

	if (aError != KErrNone) {
		TBuf<255> msg;
		msg.AppendFormat(_L("err %d: can't play\n"), aError);
		iLoadingLabel->SetTextL(msg);
		iMediaReady=EFalse;
	} else {
		iMediaReady=ETrue;
	}
	DrawNow();	
}

void CContextMediaAppPostContainer::InitControllerCompletedL(TInt aError)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("InitControllerCompletedL"));

	if (aError==KErrNone) {
		CEikButtonGroupContainer * cba = CEikButtonGroupContainer::Current();
		if( cba)  {
			cba->SetCommandSetL( R_PAUSE_STOP );
			cba->DrawNow();
		}

		if (iMediaFileType==EVideo) {
			iMediaDisplay->MakeVisible(EFalse);
			iLoadingLabel->MakeVisible(EFalse);
			iVideoEngine->DoPlayL();
		} else if (iMediaFileType==EAudio){
			iAudioEngine->DoPlayL();
		}
	} else {
		// FIX ME: popup warning note 'couldn't initialise'
	}
}

void CContextMediaAppPostContainer::RestoreOriginalCBAL()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("RestoreOriginalCBAL"));

	CEikButtonGroupContainer * cba = CEikButtonGroupContainer::Current();
	if (cba) {
		if (iView->Id()!=KPostViewId) {
			if (iMediaBrowsingOn && !iStandAlone) {
				cba->SetCommandSetL( R_AVKON_SOFTKEYS_SELECT_BACK );
			} else {
				if (!iStandAlone) {
					if (iNode != CPostStorage::MediaPoolId()) {
						cba->SetCommandSetL( R_CBA_POST_CANCEL);
					} else {
						cba->SetCommandSetL( R_AVKON_SOFTKEYS_OPTIONS_BACK);
					}
				} else {
					if (!iMediaBrowsingOn) {
						cba->SetCommandSetL( R_CBA_PUBLISH_OPTIONS );
					} else {
						cba->SetCommandSetL( R_AVKON_SOFTKEYS_OPTIONS_BACK);
					}
				}
			}
		} else {
			cba->SetCommandSetL( R_AVKON_SOFTKEYS_OPTIONS_BACK );
		}
		cba->DrawNow();
	}
}

EXPORT_C void CContextMediaAppPostContainer::Pause()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("Pause"));

	CC_TRAPD(ignore, PauseMediaFileL();)
}

EXPORT_C void CContextMediaAppPostContainer::Resume()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("Resume"));

	CC_TRAPD(ignore, ResumeMediaFileL();)
}

EXPORT_C void CContextMediaAppPostContainer::Stop()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("Stop"));

	CC_TRAPD(ignore, StopMediaFileL();)
}

CCMPost * CContextMediaAppPostContainer::get_current_post()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_current_post"));

	return iPost; // here, no need to query the storage...
}

TInt CContextMediaAppPostContainer::get_current_idx()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_current_idx"));

	return iCurrentPostIndex;
}

TInt CContextMediaAppPostContainer::get_top_idx()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_top_idx"));

	if (iCurrentPostIndex<=2) {
		return 0;
	} else {
		return (iCurrentPostIndex-2);
	}
}

void CContextMediaAppPostContainer::set_current_idx(TInt idx)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("set_current_idx"));

	iCurrentPostIndex = idx;
}

void CContextMediaAppPostContainer::set_top_idx(TInt /*idx*/)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("set_top_idx"));

	// no impl
}

TBool CContextMediaAppPostContainer::IsCurrentPlayableMedia()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("IsCurrentPlayableMedia"));

	return ( (iMediaFileType==EVideo && iVideoEngine) || (iMediaFileType==EAudio) );
}

TInt CContextMediaAppPostContainer::get_item_count()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("get_item_count"));

	return 0;
}

void CContextMediaAppPostContainer::save_contribution_to_post(TBool aConfirmed)
{
	CALLSTACKITEM_N(_CL("CContextMediaAppContainer2"), _CL("save_contribution_to_post"));

	CCMPost * aPost = get_current_post();
	if (aPost) {
		if (iComment) {
			CALLSTACKITEM_N(_CL("CContextMediaAppContainer2"), _CL("comment"));
			const TDesC& comment = iComment->RichText()->Read(0, iComment->TextLength());
			aPost->iBodyText->Zero();
			if (comment.CompareF(*iDefaultCommentBuf)!=0) {
				aPost->iBodyText->Append(comment);
			}
		}

		if (iSignature) {
			CALLSTACKITEM_N(_CL("CContextMediaAppContainer2"), _CL("signature"));
			const TDesC& sign = iSignature->RichText()->Read(0, iSignature->TextLength());
			if (sign.CompareF(*iDefaultSignatureBuf)!=0) {
				aPost->iSender.iName().Copy(sign);
				CC_TRAPD(ignore, Settings().WriteSettingL(SETTING_PUBLISH_AUTHOR, sign));
			}
		}

		if (iAutoTagArray) {
			CALLSTACKITEM_N(_CL("CContextMediaAppContainer2"), _CL("autotags"));
			aPost->iSharing()=iAutoTagArray->GetSharing();
			aPost->iIncludedTagsBitField()=iAutoTagArray->GetIncludedTagsBitField();
			if (aConfirmed) {
				CC_TRAPD(ignore, Settings().WriteSettingL(SETTING_PUBLISH_SHARING, aPost->iSharing()));
				CC_TRAP(ignore, Settings().WriteSettingL(SETTING_PUBLISH_AUTOTAGS, aPost->iIncludedTagsBitField()));
			}
		}
		if (iTags) {
			CALLSTACKITEM_N(_CL("CContextMediaAppContainer2"), _CL("tags"));
			const TDesC& tags = iTags->RichText()->Read(0, iTags->TextLength());
			aPost->iTag->Zero();
			if (tags.CompareF(*iDefaultTagsBuf)!=0) {
				aPost->iTag->Append(tags);
				if (aConfirmed) {
					CC_TRAPD(ignore, Settings().WriteSettingL(SETTING_UPLOAD_TAG, tags));
					if (iTagStorage) iTagStorage->AddFromStringAsNecessaryL(tags);
				}
			}
		}
		if (!iStandAlone) {
			TTime t; t=GetTime();
			aPost->iTimeStamp() = t;
		}
	}
}

void CContextMediaAppPostContainer::FocusNext()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("FocusNext"));

        for (int i=0;i<iControls->Count();i++) {
		if (iControls->At(i)->IsFocused()) {
			if (i+1<iControls->Count()) {
				iControls->At(i)->PrepareForFocusLossL();
				if (iAutoTags && iControls->At(i)==iAutoTags) {
					AnimateAutoTags(EFalse);
				} else {
					iControls->At(i)->OverrideColorL(EColorControlBackground, KUnfocusedBgColor);
					iControls->At(i)->OverrideColorL(EColorLabelHighlightFullEmphasis, KUnfocusedBgColor);
				}
				iControls->At(i)->SetFocus(EFalse);
				iControls->At(i)->DrawNow();
				if (iControls->At(i)==iLoadingLabel && iMediaReady) DrawMediaDisplay();
				iControls->At(i+1)->PrepareForFocusGainL();
				if (iAutoTags && iControls->At(i+1)==iAutoTags) {
					AnimateAutoTags(ETrue);
					if (iGoneUpFromAutoTags && iPost->iPresence()) iAutoTags->SetCurrentItemIndex(0);
					else iAutoTags->SetCurrentItemIndex(iAutoTagArray->MdcaCount()-1);
				} else {
					iControls->At(i+1)->OverrideColorL(EColorControlBackground, KFocusedBgColor);
					iControls->At(i+1)->OverrideColorL(EColorLabelHighlightFullEmphasis, KFocusedBgColor);
					iControls->At(i+1)->SetFocus(ETrue);
				}
				iControls->At(i+1)->SetFocus(ETrue);
				iControls->At(i+1)->DrawNow();
				
				iSBFrame->MoveThumbsBy(0, 1);
				iSBFrame->DrawScrollBarsNow();
				return;
			}
		}
	}
}

void CContextMediaAppPostContainer::FocusPrevious()
{
	CALLSTACKITEM_N(_CL("CContextMediaAppPostContainer"), _CL("FocusPrevious"));

	for (int i=iControls->Count()-1;i>=0;i--) {
		if (iControls->At(i)->IsFocused()) {
			if ((i-1)>=0 ) {
				if (iAutoTags && iControls->At(i)==iAutoTags) {
					AnimateAutoTags(EFalse);
					iGoneUpFromAutoTags=ETrue;
				} else {
					iControls->At(i)->OverrideColorL(EColorControlBackground, KUnfocusedBgColor);
					iControls->At(i)->OverrideColorL(EColorLabelHighlightFullEmphasis, KUnfocusedBgColor);
					iControls->At(i)->PrepareForFocusLossL();
				}
				iControls->At(i)->SetFocus(EFalse);

				iControls->At(i-1)->MakeVisible(ETrue);
				iControls->At(i-1)->PrepareForFocusGainL();

				iControls->At(i-1)->SetFocus(ETrue);
				if (iAutoTags && iControls->At(i-1)==iAutoTags) {
					iAutoTags->SetCurrentItemIndex(iAutoTagArray->MdcaCount()-1);
					AnimateAutoTags(ETrue);
				} else {
					iControls->At(i-1)->OverrideColorL(EColorControlBackground, KFocusedBgColor);
					iControls->At(i-1)->OverrideColorL(EColorLabelHighlightFullEmphasis, KFocusedBgColor);
					iControls->At(i-1)->DrawNow();
				}
				iControls->At(i)->DrawNow();
				if (iControls->At(i-1)==iLoadingLabel && iMediaReady) DrawMediaDisplay();
				iSBFrame->MoveThumbsBy(0, -1);
				iSBFrame->DrawScrollBarsNow();
				return;
			}
		}
	}	
}

TBool CContextMediaAppPostContainer::IsLoadingMedia()
{
	return (iLive!=0);
}

void CContextMediaAppPostContainer::ShowVolume(TInt volume)
{
	CALLSTACKITEM_N(_CL("OurOwnRichTextEditor"), _CL("ShowVolume"));
	
	TRAPD(ignore, Settings().WriteSettingL(SETTING_PLAYBACK_VOLUME, volume));

	/*CWindowGc& gc = SystemGc();

	TRect aRect = TRect(TPoint(0,0), TSize(40,40));
	gc.SetPenStyle(CGraphicsContext::ESolidPen);
	gc.SetPenColor(KRgbYellow);
	gc.SetBrushStyle(CGraphicsContext::ENullBrush);
	const CFont* fontUsed = iEikonEnv->DenseFont();
	gc.UseFont(fontUsed);
	TInt baseline = aRect.Height() - 2 * fontUsed->AscentInPixels();
	TBuf<10> buf;
	buf.AppendFormat(_L("vol.%d"), volume);
	gc.DrawText(buf, aRect, baseline, CGraphicsContext::ECenter);
*/
	iVolumeDisplayTimeOut->Reset();
	TBuf<10> buf;
	buf.AppendFormat(_L("vol.%d"), volume);
	iVolumeIndicator->SetTextL(buf);
	iVolumeIndicator->DrawDeferred();
	iVolumeDisplayTimeOut->Wait(1);

}


///--------------------------------------------------------------------------------------------

void OurOwnRichTextEditor::HandleTextChangedL()
{
	CALLSTACKITEM_N(_CL("OurOwnRichTextEditor"), _CL("HandleTextChangedL"));

	CEikRichTextEditor::HandleTextChangedL();

	CParaFormat paraFormat;
	TParaFormatMask paraFormatMask;

	RichText()->GetParaFormatL(&paraFormat, paraFormatMask, 0, TextLength());
	paraFormatMask.SetAttrib(EAttLineSpacingControl);
	paraFormatMask.SetAttrib(EAttLineSpacing);
	paraFormatMask.SetAttrib(EAttAlignment);
	paraFormat.iLineSpacingControl = CParaFormat::ELineSpacingExactlyInTwips;
	paraFormat.iLineSpacingInTwips = 115;
	paraFormat.iHorizontalAlignment = CParaFormat::ELeftAlign;
	RichText()->ApplyParaFormatL(&paraFormat, paraFormatMask, 0, TextLength());
}

void OurOwnRichTextEditor::Reset()
{
	CALLSTACKITEM_N(_CL("OurOwnRichTextEditor"), _CL("Reset"));

	RichText()->Reset();
	CC_TRAPD(err, RichText()->InsertL(0, *iDefaultValue));
}

void OurOwnRichTextEditor::PrepareForFocusGainL()
{
	CALLSTACKITEM_N(_CL("OurOwnRichTextEditor"), _CL("PrepareForFocusGainL"));

	SetCursorPosL(0, EFalse);
	if (TextLength()==0) {
		Reset();
		HandleTextChangedL();
	}
	SetCursorPosL(TextLength(), ETrue);
}

void OurOwnRichTextEditor::PrepareForFocusLossL()
{
	CALLSTACKITEM_N(_CL("OurOwnRichTextEditor"), _CL("PrepareForFocusLossL"));
	if (TextLength()==0) {
		SetCursorPosL(0, EFalse);
		Reset();
		HandleTextChangedL();
		SetCursorPosL(TextLength(), ETrue);
	}
}