#include "break.h"
/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "statusview.h"
#include <akntitle.h>
#include <eikspane.h>
#ifndef FLICKR
#include <context_log.rsg>
#else
#include <contextflickr.rsg>
#endif

// INCLUDE FILES
#include  <aknviewappui.h>
#include  <avkon.hrh>
#include  "StatusView.h"
#include  "context_logContainer.h" 
#include  "Context_log.hrh"
#include  "symbian_auto_ptr.h"
#include "view_ids.h"
#include <eikdll.h>
const TUid KUidPhone = { 0x100058b3 };

// ---------------------------------------------------------
// CStatusView::ConstructL(const TRect& aRect)
// EPOC two-phased constructor
// ---------------------------------------------------------
//

CStatusView* CStatusView::NewL(MApp_context& Context)
{
	CALLSTACKITEM2_N(_CL("CStatusView"), _CL("NewL"), &Context);

	auto_ptr<CStatusView> ret(new (ELeave) CStatusView(Context));
	ret->ConstructL();
	return ret.release();
}

void CStatusView::ConstructL()
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("ConstructL"));

#ifndef FLICKR
	BaseConstructL( R_CL_STATUSVIEW );
#else
	BaseConstructL( R_CL_FLICKR_STATUSVIEW );
#endif
	iContainer = new (ELeave) CContext_logContainer(AppContext());
	iContainer->MakeVisible(EFalse);
	iContainer->SetMopParent(this);
	iContainer->ConstructL( ClientRect() );
}

// ---------------------------------------------------------
// CStatusView::~CStatusView()
// ?implementation_description
// ---------------------------------------------------------
//
CStatusView::~CStatusView()
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("~CStatusView"));

	if ( iContainer && iActive )
        {
		AppUi()->RemoveFromViewStack( *this, iContainer );
        }
	
	delete iContainer;
}

// ---------------------------------------------------------
// TUid CStatusView::Id()
// ?implementation_description
// ---------------------------------------------------------
//
TUid CStatusView::Id() const
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("Id"));

	return KStatusView;
}

// ---------------------------------------------------------
// CStatusView::HandleCommandL(TInt aCommand)
// ?implementation_description
// ---------------------------------------------------------
//
void CStatusView::HandleCommandL(TInt aCommand)
{   
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("HandleCommandL"));

	AppUi()->HandleCommandL(aCommand);
}

// ---------------------------------------------------------
// CStatusView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CStatusView::HandleClientRectChange()
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("HandleClientRectChange"));

	if ( iContainer )
        {
		iContainer->SetRect( ClientRect() );
        }
}

// ---------------------------------------------------------
// CStatusView::DoActivateL(...)
// ?implementation_description
// ---------------------------------------------------------
//
void CStatusView::DoActivateL(
				   const TVwsViewId& /*aPrevViewId*/,TUid /*aCustomMessageId*/,
				   const TDesC8& /*aCustomMessage*/)
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("DoActivateL"));

	iActive=true;
	iContainer->MakeVisible(ETrue);
	AppUi()->AddToStackL( *this, iContainer );
	CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
	CAknTitlePane* tp=(CAknTitlePane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidTitle));
	tp->SetText(CEikonEnv::Static()->AllocReadResourceL(R_CAPTION));
	if ( iNextViewId != TVwsViewId() ) {
		TVwsViewId v = iNextViewId;
		iNextViewId= TVwsViewId();

		{
			// it tends not to be safe to try to re-activate
			// the media capture apps if they've been shut down
			
			RWsSession& ws=CEikonEnv::Static()->WsSession();
			TApaTaskList tl(ws);
			TApaTask app_task=tl.FindApp(v.iAppUid);
			if (! app_task.Exists() ) {
				// so activate the phone app instead
				TApaTask app_task=tl.FindApp(KUidPhone);
				if (app_task.Exists()) app_task.BringToForeground();
				goto done_activate;
			}
		}
		if (v.iAppUid == KCameraUid) {
			CC_TRAPD(err, ActivateViewL(TVwsViewId(KCameraUid, KCameraViewUid)));
		} else if (v.iAppUid == KCamera2Uid) {
			CC_TRAPD(err, ActivateViewL(TVwsViewId(KCamera2Uid, KCamera2ViewUid)));
		} else if (v.iAppUid == KCamera3Uid) {
			// the camera on N70 will hang if the slider's been closed and we
			//   try to reactivate it 
			//Reporting().UserErrorLog(_L("starting camera3"));
			auto_ptr<CApaCommandLine> cmd(CApaCommandLine::NewL(_L("z:\\system\\apps\\cammojave\\cammojave.app")));
			cmd->SetCommandL(EApaCommandRun);
			CC_TRAPD(err, EikDll::StartAppL(*cmd));			
		} else if (v.iAppUid == KVideoUid) {
			CC_TRAPD(err, ActivateViewL(TVwsViewId(KVideoUid, KVideoViewUid)));
                } else if (v.iAppUid == KRecorderUid) {
			auto_ptr<CApaCommandLine> cmd(CApaCommandLine::NewL(_L("z:\\system\\apps\\voicerecorder\\voicerecorder.app")));
			cmd->SetCommandL(EApaCommandRun);
			CC_TRAPD(err, EikDll::StartAppL(*cmd));			
		} else {
			//Reporting().UserErrorLog(_L("activating previous"));
			CC_TRAPD(err, ActivateViewL(v));
		}
done_activate:
		;

	}
}

// ---------------------------------------------------------
// CStatusView::HandleCommandL(TInt aCommand)
// ?implementation_description
// ---------------------------------------------------------
//
void CStatusView::DoDeactivate()
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("DoDeactivate"));

	iActive=false;
	AppUi()->RemoveFromViewStack( *this, iContainer );
	iContainer->MakeVisible(EFalse);
}

void CStatusView::DynInitMenuPaneL(
		 TInt aResourceId,CEikMenuPane* aMenuPane)
{
	CALLSTACKITEM_N(_CL("CStatusView"), _CL("DynInitMenuPaneL"));

	AppUi()->DynInitMenuPaneL(aResourceId, aMenuPane);
}

// End of File


